# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/configuration_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_configurationDialog(object):

    def setupUi(self, configurationDialog):
        configurationDialog.setObjectName("configurationDialog")
        configurationDialog.resize(585, 454)
        configurationDialog.setModal(True)
        self.gridlayout = QtWidgets.QGridLayout(configurationDialog)
        self.gridlayout.setObjectName("gridlayout")
        self.splitter = QtWidgets.QSplitter(configurationDialog)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayout = QtWidgets.QWidget(self.splitter)
        self.verticalLayout.setObjectName("verticalLayout")
        self.vboxlayout = QtWidgets.QVBoxLayout(self.verticalLayout)
        self.vboxlayout.setContentsMargins(0, 0, 0, 0)
        self.vboxlayout.setSpacing(4)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiTitleLabel = QtWidgets.QLabel(self.verticalLayout)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.uiTitleLabel.setFont(font)
        self.uiTitleLabel.setFrameShape(QtWidgets.QFrame.Box)
        self.uiTitleLabel.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.uiTitleLabel.setTextFormat(QtCore.Qt.PlainText)
        self.uiTitleLabel.setObjectName("uiTitleLabel")
        self.vboxlayout.addWidget(self.uiTitleLabel)
        self.uiConfigStackedWidget = QtWidgets.QStackedWidget(self.verticalLayout)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConfigStackedWidget.sizePolicy().hasHeightForWidth())
        self.uiConfigStackedWidget.setSizePolicy(sizePolicy)
        self.uiConfigStackedWidget.setFrameShape(QtWidgets.QFrame.Box)
        self.uiConfigStackedWidget.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.uiConfigStackedWidget.setObjectName("uiConfigStackedWidget")
        self.uiEmptyPageWidget = QtWidgets.QWidget()
        self.uiEmptyPageWidget.setObjectName("uiEmptyPageWidget")
        self.vboxlayout1 = QtWidgets.QVBoxLayout(self.uiEmptyPageWidget)
        self.vboxlayout1.setContentsMargins(0, 4, 0, 0)
        self.vboxlayout1.setSpacing(0)
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.uiConfigStackedWidget.addWidget(self.uiEmptyPageWidget)
        self.vboxlayout.addWidget(self.uiConfigStackedWidget)
        self.gridlayout.addWidget(self.splitter, 0, 0, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(configurationDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridlayout.addWidget(self.uiButtonBox, 1, 0, 1, 1)

        self.retranslateUi(configurationDialog)
        self.uiConfigStackedWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(configurationDialog)

    def retranslateUi(self, configurationDialog):
        _translate = QtCore.QCoreApplication.translate
        configurationDialog.setWindowTitle(_translate("configurationDialog", "Configuration"))
        self.uiTitleLabel.setText(_translate("configurationDialog", "Configuration"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import os
import json

from gns3.topology_check import getTopologyValidationErrors


schemas_directory = os.path.join(os.path.dirname(os.path.realpath(__file__)), "schemas")


@pytest.mark.parametrize("file", os.listdir(schemas_directory))
def test_load(file):
    with open(os.path.join(schemas_directory, file)) as f:
        topology = json.load(f)
        errors = getTopologyValidationErrors(topology)
        assert errors is None, errors


@pytest.mark.parametrize("file", os.listdir(schemas_directory))
def test_load_invalid(file):
    with open(os.path.join(schemas_directory, file)) as f:
        topology = json.load(f)
        if len(topology["topology"].get("nodes", [])) > 0:

            # Simulate an error
            del topology["topology"]["nodes"][0]["properties"]["name"]

            errors = getTopologyValidationErrors(topology)
            assert errors is not None, errors



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for VM classes.
"""

import os
from gns3.servers import Servers

from .node import Node

import logging
log = logging.getLogger(__name__)


class VM(Node):

    def __init__(self, module, server, project):

        super().__init__(module, server, project)

        self._vm_id = None
        self._vm_directory = None
        self._command_line = None

    def consoleCommand(self):
        """
        :returns: The console command for this host
        """
        from .main_window import MainWindow
        general_settings = MainWindow.instance().settings()

        console_type = self.consoleType()
        if console_type == "serial":
            return general_settings["serial_console_command"]
        elif console_type == "vnc":
            return general_settings["vnc_console_command"]
        return general_settings["telnet_console_command"]

    def consoleType(self):
        """
        Get the console type (serial, telnet or VNC)
        """
        console_type = "telnet"
        if hasattr(self, "serialConsole") and self.serialConsole():
            return "serial"
        if "console_type" in self.settings():
            return self.settings()["console_type"]
        return console_type

    def vm_id(self):
        """
        Return the ID of this device

        :returns: identifier (string)
        """

        return self._vm_id

    def vmDir(self):
        """
        Return the working directory of the VM

        :returns: identifier (string)
        """

        return self._vm_directory

    def commandLine(self):
        """
        Return the command line used to run the VM

        :returns: identifier (string)
        """

        return self._command_line

    def delete(self):
        """
        Deletes this VM instance.
        """

        log.debug("{} is being deleted".format(self.name()))
        # first delete all the links attached to this node
        self.delete_links_signal.emit()
        if self._vm_id and self._server.connected():
            self.httpDelete("/{prefix}/vms/{vm_id}".format(prefix=self.URL_PREFIX, vm_id=self._vm_id), self._deleteCallback)
        else:
            self.deleted_signal.emit()
            self._module.removeNode(self)

    def _deleteCallback(self, result, error=False, **kwargs):
        """
        Callback for delete.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        log.info("{} has been deleted".format(self.name()))
        self.deleted_signal.emit()
        self._module.removeNode(self)

    def start(self):
        """
        Starts this VM instance.
        """

        if self.status() == Node.started:
            log.debug("{} is already running".format(self.name()))
            return

        log.debug("{} is starting".format(self.name()))
        self.httpPost("/{prefix}/vms/{vm_id}/start".format(prefix=self.URL_PREFIX, vm_id=self._vm_id), self._startCallback, progressText="{} is starting".format(self.name()))

    def _startCallback(self, result, error=False, **kwargs):
        """
        Callback for start.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while starting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has started".format(self.name()))
            self.setStatus(Node.started)
            if result:
                self._updateCallback(result)

    def _setupCallback(self, result, error=False, **kwargs):
        """
        Callback for setup.

        :param result: server response
        :param error: indicates an error (boolean)
        :returns: Boolean success or not
        """

        if error:
            log.error("error while setting up {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return False

        self._vm_id = result["vm_id"]
        if not self._vm_id:
            self.error_signal.emit(self.id(), "returned ID from server is null")
            return False

        if "vm_directory" in result:
            self._vm_directory = result["vm_directory"]

        if "command_line" in result:
            self._command_line = result["command_line"]

        # update the settings using the defaults sent by the server
        for name, value in result.items():
            if name in self._settings and self._settings[name] != value:
                log.info("{} setting up and updating {} from '{}' to '{}'".format(self.name(),
                                                                                  name,
                                                                                  self._settings[name],
                                                                                  value))
                self._settings[name] = value
        return True

    def _updateCallback(self, result, error=False, **kwargs):
        """
        Callback for update.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return False

        if "command_line" in result:
            self._command_line = result["command_line"]

        return True

    def stop(self):
        """
        Stops this VM instance.
        """

        if self.status() == Node.stopped:
            log.debug("{} is already stopped".format(self.name()))
            return

        log.debug("{} is stopping".format(self.name()))
        self.httpPost("/{prefix}/vms/{vm_id}/stop".format(prefix=self.URL_PREFIX, vm_id=self._vm_id), self._stopCallback, progressText="{} is stopping".format(self.name()))

    def _stopCallback(self, result, error=False, **kwargs):
        """
        Callback for stop.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while stopping {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has stopped".format(self.name()))
            self.setStatus(Node.stopped)

    def reload(self):
        """
        Reloads this VM instance.
        """

        log.debug("{} is being reloaded".format(self.name()))
        self.httpPost("/{prefix}/vms/{vm_id}/reload".format(prefix=self.URL_PREFIX, vm_id=self._vm_id), self._reloadCallback)

    def _reloadCallback(self, result, error=False, **kwargs):
        """
        Callback for reload.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while suspending {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has reloaded".format(self.name()))

    def addNIO(self, port, nio):
        """
        Adds a new NIO on the specified port for this VM instance.

        :param port: Port instance
        :param nio: NIO instance
        """
        params = self.getNIOInfo(nio)
        log.debug("{} is adding an {}: {}".format(self.name(), nio, params))
        self.httpPost("/{prefix}/vms/{vm_id}/adapters/{adapter}/ports/{port}/nio".format(
            adapter=port.adapterNumber(),
            port=port.portNumber(),
            prefix=self.URL_PREFIX,
            vm_id=self._vm_id),
            self._addNIOCallback,
            context={"port_id": port.id()},
            body=params)

    def _addNIOCallback(self, result, error=False, context={}, **kwargs):
        """
        Callback for addNIO.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while adding a NIO for {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            self.nio_cancel_signal.emit(self.id())
        else:
            self.nio_signal.emit(self.id(), context["port_id"])

    def deleteNIO(self, port):
        """
        Deletes an NIO from the specified port on this instance

        :param port: Port instance
        """

        log.debug("{} is deleting an NIO".format(self.name()))
        self.httpDelete("/{prefix}/vms/{vm_id}/adapters/{adapter}/ports/{port}/nio".format(
            adapter=port.adapterNumber(),
            prefix=self.URL_PREFIX,
            port=port.portNumber(),
            vm_id=self._vm_id),
            self._deleteNIOCallback)

    def _deleteNIOCallback(self, result, error=False, **kwargs):
        """
        Callback for deleteNIO.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("Error while deleting NIO {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return

        log.debug("{} has deleted a NIO: {}".format(self.name(), result))

    def _readBaseConfig(self, config_path):
        """
        Returns a base config content.

        :param config_path: path to the configuration file.

        :returns: config content
        """

        if config_path is None or len(config_path.strip()) == 0:
            return None

        if not os.path.isabs(config_path):
            config_path = os.path.join(Servers.instance().localServerSettings()["configs_path"], config_path)

        if not os.path.isfile(config_path):
            return None

        try:
            with open(config_path, "rb") as f:
                log.info("Opening configuration file: {}".format(config_path))
                config = f.read().decode("utf-8")
                config = config.replace('\r', "")
                return config
        except OSError as e:
            self.error_signal.emit(self.id(), "Could not read configuration file {}: {}".format(config_path, e))
            return None
        except UnicodeDecodeError as e:
            self.error_signal.emit(self.id(), "Invalid configuration file {}: {}".format(config_path, e))
            return None
        return ""

    def dump(self):
        """
        Returns a representation of this device.
        (to be saved in a topology file).

        :returns: representation of the node (dictionary)
        """

        device = {
            "id": self.id(),
            "type": self.__class__.__name__,
            "description": str(self),
            "properties": {},
            "server_id": self._server.id()
        }

        # add the ports
        if self._ports:
            # In 1.4.2dev1 we track an issue about duplicate port name
            # https://github.com/GNS3/gns3-gui/issues/992
            initialized_port_name = set()

            ports = device["ports"] = []
            for port in self._ports:
                if port.name() in initialized_port_name:
                    msg = "Duplicate port name {} in {}.".format(port.name(), self.name())
                    print(msg)
                    log.error(msg)
                else:
                    ports.append(port.dump())
                    initialized_port_name.add(port.name())

        return device

    def load(self, node_info):
        """
        Loads a device representation
        (from a topology file).

        :param node_info: representation of the node (dictionary)
        """

        self._loading = True
        self._node_info = node_info
        self.loaded_signal.connect(self._updatePortSettings)

    def openConsole(self, command=None, aux=False):
        if command is None:
            command = self.consoleCommand()

        if hasattr(self, "serialConsole") and self.serialConsole():
            from .serial_console import serialConsole
            serialConsole(self.name(), self.serialPipe(), command)
            return

        if aux:
            console_port = self.auxConsole()
            if console_port is None:
                raise ValueError("AUX console port not allocated for {}".format(self.name()))
        else:
            console_port = self.console()

        console_type = "telnet"
        if "console_type" in self.settings():
            console_type = self.settings()["console_type"]
        if console_type == "telnet":
            from .telnet_console import nodeTelnetConsole
            nodeTelnetConsole(self.name(), self.server(), console_port, command)
        elif console_type == "vnc":
            from .vnc_console import vncConsole
            vncConsole(self.server().host(), console_port, command)

    def _updatePortSettings(self):
        """
        Updates port settings when loading a topology.
        """

        self.loaded_signal.disconnect(self._updatePortSettings)

        # assign the correct names and IDs to the ports
        if "ports" in self._node_info:
            ports = self._node_info["ports"]

            port_initialized = set()

            for topology_port in ports:
                for port in self._ports:
                    if topology_port["port_number"] == port.portNumber():
                        # If the adapter is missing we consider that adapter_number == port_number
                        adapter_number = topology_port.get("adapter_number", topology_port["port_number"])
                        if port.adapterNumber() is None or adapter_number == port.adapterNumber() or topology_port.get("slot_number", None) == port.adapterNumber():

                            if port in port_initialized:
                                msg = "Topology corrupted port {} already exists for {}".format(port, self.name())
                                print(msg)
                                log.error(msg)
                            else:
                                port.setName(topology_port["name"])
                                port.setId(topology_port["id"])
                                port_initialized.add(port)

        # now we can set the node as initialized and trigger the created signal
        self.setInitialized(True)
        log.info("{} has been loaded".format(self.name()))
        self.created_signal.emit(self.id())
        self._module.addNode(self)
        self._loading = False
        self._node_info = None



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for VirtualBox VMs.
"""

from gns3.qt import QtGui, QtWidgets
from gns3.dialogs.node_properties_dialog import ConfigurationError
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.node import Node

from ..ui.virtualbox_vm_configuration_page_ui import Ui_virtualBoxVMConfigPageWidget


class VirtualBoxVMConfigurationPage(QtWidgets.QWidget, Ui_virtualBoxVMConfigPageWidget):

    """
    QWidget configuration page for VirtualBox VMs.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)
        self.uiAdapterTypesComboBox.clear()
        self.uiAdapterTypesComboBox.addItems(["PCnet-PCI II (Am79C970A)",
                                              "PCNet-FAST III (Am79C973)",
                                              "Intel PRO/1000 MT Desktop (82540EM)",
                                              "Intel PRO/1000 T Server (82543GC)",
                                              "Intel PRO/1000 MT Server (82545EM)",
                                              "Paravirtualized Network (virtio-net)"])

        # TODO: finish VM name change
        self.uiVMListLabel.hide()
        self.uiVMListComboBox.hide()

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the VirtualBox VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        if not group:

            # set the device name
            if "name" in settings:
                self.uiNameLineEdit.setText(settings["name"])
            else:
                self.uiNameLabel.hide()
                self.uiNameLineEdit.hide()

            if "console" in settings:
                self.uiConsolePortSpinBox.setValue(settings["console"])
            else:
                self.uiConsolePortLabel.hide()
                self.uiConsolePortSpinBox.hide()

            if "linked_base" in settings:
                self.uiBaseVMCheckBox.setChecked(settings["linked_base"])
            else:
                self.uiBaseVMCheckBox.hide()

        else:
            self.uiNameLabel.hide()
            self.uiNameLineEdit.hide()
            self.uiConsolePortLabel.hide()
            self.uiConsolePortSpinBox.hide()
            self.uiVMListLabel.hide()
            self.uiVMListComboBox.hide()

        if not node:
            # these are template settings

            # rename the label from "Name" to "Template name"
            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            # load the category
            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)

            self.uiPortNameFormatLineEdit.setText(settings["port_name_format"])
            self.uiPortSegmentSizeSpinBox.setValue(settings["port_segment_size"])
            self.uiFirstPortNameLineEdit.setText(settings["first_port_name"])
        else:
            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()
            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()
            self.uiCategoryComboBox.hide()
            self.uiPortNameFormatLabel.hide()
            self.uiPortNameFormatLineEdit.hide()
            self.uiPortSegmentSizeLabel.hide()
            self.uiPortSegmentSizeSpinBox.hide()
            self.uiFirstPortNameLabel.hide()
            self.uiFirstPortNameLineEdit.hide()

        self.uiAdaptersSpinBox.setValue(settings["adapters"])
        index = self.uiAdapterTypesComboBox.findText(settings["adapter_type"])
        if index != -1:
            self.uiAdapterTypesComboBox.setCurrentIndex(index)
        self.uiUseAnyAdapterCheckBox.setChecked(settings["use_any_adapter"])
        self.uiVMRamSpinBox.setValue(settings["ram"])
        self.uiHeadlessModeCheckBox.setChecked(settings["headless"])
        self.uiACPIShutdownCheckBox.setChecked(settings["acpi_shutdown"])
        self.uiEnableConsoleCheckBox.setChecked(settings["enable_remote_console"])

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the VirtualBox VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        # these settings cannot be shared by nodes and updated
        # in the node properties dialog.
        if not group:

            if "name" in settings:
                name = self.uiNameLineEdit.text()
                if not name:
                    QtWidgets.QMessageBox.critical(self, "Name", "VirtualBox name cannot be empty!")
                else:
                    settings["name"] = name

            if "console" in settings:
                settings["console"] = self.uiConsolePortSpinBox.value()

            if "linked_base" in settings:
                settings["linked_base"] = self.uiBaseVMCheckBox.isChecked()

            settings["enable_remote_console"] = self.uiEnableConsoleCheckBox.isChecked()

        else:
            del settings["name"]
            del settings["console"]
            del settings["enable_remote_console"]

        if not node:
            # these are template settings

            # save the default name format
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            pixmap = QtGui.QPixmap(symbol_path)
            if pixmap.isNull():
                QtWidgets.QMessageBox.critical(self, "Symbol", "Invalid file or format not supported")
            else:
                settings["symbol"] = symbol_path

            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())
            port_name_format = self.uiPortNameFormatLineEdit.text()
            if '{0}' not in port_name_format and '{port0}' not in port_name_format and '{port1}' not in port_name_format:
                QtWidgets.QMessageBox.critical(self, "Port name format", "The format must contain at least {0}, {port0} or {port1}")
            else:
                settings["port_name_format"] = self.uiPortNameFormatLineEdit.text()

            port_segment_size = self.uiPortSegmentSizeSpinBox.value()
            if port_segment_size and '{1}' not in port_name_format and '{segment0}' not in port_name_format and '{segment1}' not in port_name_format:
                QtWidgets.QMessageBox.critical(self, "Port name format", "If the segment size is not 0, the format must contain {1}, {segment0} or {segment1}")
            else:
                settings["port_segment_size"] = port_segment_size

            settings["first_port_name"] = self.uiFirstPortNameLineEdit.text().strip()

        settings["ram"] = self.uiVMRamSpinBox.value()
        settings["adapter_type"] = self.uiAdapterTypesComboBox.currentText()
        settings["headless"] = self.uiHeadlessModeCheckBox.isChecked()
        settings["acpi_shutdown"] = self.uiACPIShutdownCheckBox.isChecked()
        settings["use_any_adapter"] = self.uiUseAnyAdapterCheckBox.isChecked()

        adapters = self.uiAdaptersSpinBox.value()
        if node:
            if settings["adapters"] != adapters:
                # check if the adapters settings have changed
                node_ports = node.ports()
                for node_port in node_ports:
                    if not node_port.isFree():
                        QtWidgets.QMessageBox.critical(self, node.name(), "Changing the number of adapters while links are connected isn't supported yet! Please delete all the links first.")
                        raise ConfigurationError()
        settings["adapters"] = adapters



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import base64
import os

from . import QtCore
from . import QtSvg
from . import QtGui


class QImageSvgRenderer(QtSvg.QSvgRenderer):
    """
    Renderer pixmap and svg to SVG item
    """
    def __init__(self, path):
        super().__init__()
        super().load(path)

        # If we can't render a SVG we load and base64 the image to create a SVG
        if not self.isValid() and os.path.exists(path):
            image = QtGui.QImage(path)
            data = QtCore.QByteArray()
            buf = QtCore.QBuffer(data)
            image.save(buf, 'PNG')
            xml = """<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
<image width="{width}" height="{height}" xlink:href="data:image/png;base64,{data}"/>
</svg>""".format(data=bytes(data.toBase64()).decode(),
                width=image.rect().width(),
                height=image.rect().height())
            super().load(xml.encode())




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Thread to wait for an IOS image to be decompressed.
"""

import zipfile
import zlib

from gns3.qt import QtCore
from .decompress_ios import decompressIOS


class DecompressIOSWorker(QtCore.QObject):

    """
    Thread to decompress an IOS image.

    :param ios_image: IOS image path
    :param destination_file: destination path for the decompressed IOS image
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    def __init__(self, ios_image, destination_file):

        super().__init__()
        self._is_running = False
        self._ios_image = ios_image
        self._destination_file = destination_file

    def run(self):
        """
        Thread starting point.
        """

        self._is_running = True
        try:
            decompressIOS(self._ios_image, self._destination_file)
        except (zipfile.BadZipFile, zlib.error) as e:
            self.error.emit("File {} is corrupted {}".format(self._ios_image, e), True)
            return
        except (OSError, MemoryError) as e:
            self.error.emit("Could not decompress {}: {}".format(self._ios_image, e), True)
            return

        # IOS image has successfully been decompressed
        self.finished.emit()

    def cancel(self):
        """
        Cancel this worker.
        """

        if not self:
            return
        self._is_running = False



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Keeps track of all the local and remote servers and their settings.
"""

import sys
import os
import shlex
import signal
import urllib
import shutil
import string
import random
import socket
import subprocess
import binascii
import stat
import struct
import psutil

from .qt import QtNetwork, QtWidgets, QtCore
from .network_client import getNetworkUrl
from .local_config import LocalConfig
from .settings import SERVERS_SETTINGS
from .local_server_config import LocalServerConfig
from .progress import Progress
from .utils.sudo import sudo

from collections import OrderedDict

import logging
log = logging.getLogger(__name__)


class Servers(QtCore.QObject):

    """
    Server management class.
    """

    server_added_signal = QtCore.Signal(str)

    def __init__(self):

        super().__init__()
        self._settings = {}
        self._local_server = None
        self._vm_server = None
        self._remote_servers = {}
        self._local_server_path = ""
        self._local_server_auto_start = True
        self._local_server_allow_console_from_anywhere = False
        self._local_server_process = None
        self._network_manager = QtNetwork.QNetworkAccessManager()
        self._network_manager.sslErrors.connect(self._handleSslErrors)
        self._remote_server_iter_pos = 0
        self._loadSettings()
        self._pid_path = os.path.join(LocalConfig.configDirectory(), "gns3_server.pid")
        self.registerLocalServer()

    def servers(self):
        """
        Return the list of all servers, remote, vm and local
        """
        servers = list(self._remote_servers.values())
        if self._local_server:
            servers.append(self._local_server)
        if self._vm_server:
            servers.append(self._vm_server)
        return servers

    def registerLocalServer(self):
        """
        Register a new local server.
        """

        local_server_settings = self._settings["local_server"]
        host = local_server_settings["host"]
        port = local_server_settings["port"]
        user = local_server_settings["user"]
        password = local_server_settings["password"]
        self._local_server = self.getNetworkClientInstance({"host": host, "port": port, "user": user, "password": password},
                                                      self._network_manager)
        self._local_server.setLocal(True)
        self.server_added_signal.emit("local")
        log.info("New local server connection {} registered".format(self._local_server.url()))


    @staticmethod
    def _findLocalServer(self):
        """
        Finds the local server path.

        :return: path to the local server
        """

        local_server_path = shutil.which("gns3server")

        if local_server_path is None:
            return ""
        return os.path.abspath(local_server_path)

    @staticmethod
    def _findUbridge(self):
        """
        Finds the ubridge executable path.

        :return: path to the ubridge
        """

        ubridge_path = shutil.which("ubridge")

        if ubridge_path is None:
            return ""
        path = os.path.abspath(ubridge_path)
        return path

    def _checkUbridgePermissions(self):
        """
        Checks that uBridge can interact with network interfaces.
        """

        path = self._settings["local_server"]["ubridge_path"]

        if not path or len(path) == 0 or not os.path.exists(path):
            return False

        if sys.platform.startswith("win"):
            # do not check anything on Windows
            return True

        if os.geteuid() == 0:
            # we are root, so we should have privileged access.
            return True

        from .main_window import MainWindow
        main_window = MainWindow.instance()

        request_setuid = False
        if sys.platform.startswith("linux"):
            # test if the executable has the CAP_NET_RAW capability (Linux only)
            try:
                if "security.capability" in os.listxattr(path):
                    caps = os.getxattr(path, "security.capability")
                    # test the 2nd byte and check if the 13th bit (CAP_NET_RAW) is set
                    if not struct.unpack("<IIIII", caps)[1] & 1 << 13:
                        proceed = QtWidgets.QMessageBox.question(
                            main_window,
                            "uBridge",
                            "uBridge requires CAP_NET_RAW capability to interact with network interfaces. Set the capability to uBridge?",
                            QtWidgets.QMessageBox.Yes,
                            QtWidgets.QMessageBox.No)
                        if proceed == QtWidgets.QMessageBox.Yes:
                            sudo(["setcap", "cap_net_admin,cap_net_raw=ep"])
                else:
                    # capabilities not supported
                    request_setuid = True
            except OSError as e:
                QtWidgets.QMessageBox.critical(main_window, "uBridge", "Can't set CAP_NET_RAW capability to uBridge {}: {}".format(path, str(e)))
                return False

        if sys.platform.startswith("darwin") or request_setuid:
            try:
                if os.stat(path).st_uid != 0 or not os.stat(path).st_mode & stat.S_ISUID:
                    proceed = QtWidgets.QMessageBox.question(
                        main_window,
                        "uBridge",
                        "uBridge requires root permissions to interact with network interfaces. Set root permissions to uBridge?",
                        QtWidgets.QMessageBox.Yes,
                        QtWidgets.QMessageBox.No)
                    if proceed == QtWidgets.QMessageBox.Yes:
                        sudo(["chmod", "4755", path])
                        sudo(["chown", "root", path])
            except OSError as e:
                QtWidgets.QMessageBox.critical(main_window, "uBridge", "Can't set root permissions to uBridge {}: {}".format(path, str(e)))
                return False
        return True

    def _handleSslErrors(self, reply, errorList):
        """
        Called when an SSL error occur
        """

        url = reply.url().toDisplayString()
        server = self.getServerFromString(url)
        certificate = binascii.hexlify(errorList[0].certificate().digest()).decode('utf-8')
        if server.acceptInsecureCertificate() == certificate:
            reply.ignoreSslErrors()
            return

        with Progress.instance().context(enable=False):
            from .main_window import MainWindow
            main_window = MainWindow.instance()
            proceed = QtWidgets.QMessageBox.warning(
                main_window,
                "SSL Error",
                "SSL certificate for:\n {} is invalid or someone is trying to intercept the communication.\nContinue?".format(reply.url().toDisplayString()),
                QtWidgets.QMessageBox.Yes,
                QtWidgets.QMessageBox.No)

            if proceed == QtWidgets.QMessageBox.Yes:
                server.setAcceptInsecureCertificate(certificate)
                self._saveSettings()
                reply.ignoreSslErrors()
                log.info("SSL error ignored for %s", url)

    def _passwordGenerate(self):
        """
        Generate a random password
        """
        return ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(64))

    def _loadSettings(self):
        """
        Loads the server settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings("Servers", SERVERS_SETTINGS)

        local_server_settings = self._settings["local_server"]
        if not os.path.exists(local_server_settings["path"]):
            local_server_settings["path"] = self._findLocalServer(self)

        if not os.path.exists(local_server_settings["ubridge_path"]):
            local_server_settings["ubridge_path"] = self._findUbridge(self)

        for remote_server in self._settings["remote_servers"]:
            self._addRemoteServer(protocol=remote_server.get("protocol", "http"),
                                  host=remote_server["host"],
                                  port=remote_server["port"],
                                  ram_limit=remote_server.get("ram_limit", 0),
                                  user=remote_server.get("user", None),
                                  password=remote_server.get("password", None),
                                  accept_insecure_certificate=remote_server.get("accept_insecure_certificate", False))

        changed = False
        if "user" not in local_server_settings or len(local_server_settings["user"]) == 0:
            local_server_settings["user"] = self._passwordGenerate()
            local_server_settings["password"] = self._passwordGenerate()
            changed = True

        # For 1.3 compatibity old LocalServer section
        local_server = LocalConfig.instance().loadSectionSettings("LocalServer", {})
        if "auth" in local_server:
            local_server["auth"] = local_server_settings["auth"]
            local_server["user"] = local_server_settings["user"]
            local_server["password"] = local_server_settings["password"]
            LocalConfig.instance().saveSectionSettings("LocalServer", local_server)
            changed = True

        # WARNING: This operation should be a the end of the method otherwise you save a partial config
        if changed:
            self._saveSettings()

    def _saveSettings(self):
        """
        Saves the server settings to a persistent settings file.
        """

        # save the remote servers
        self._settings["remote_servers"] = []
        for server in self._remote_servers.values():
            settings = server.settings()
            settings["url"] = server.url()
            self._settings["remote_servers"].append(settings)

        # save the settings
        LocalConfig.instance().saveSectionSettings("Servers", self._settings)

        # save some settings to the local server config files
        local_server_settings = self._settings["local_server"]
        server_settings = OrderedDict([
            ("host", local_server_settings["host"]),
            ("port", local_server_settings["port"]),
            ("ubridge_path", local_server_settings["ubridge_path"]),
            ("auth", local_server_settings.get("auth", False)),
            ("user", local_server_settings.get("user", "")),
            ("password", local_server_settings.get("password", "")),
            ("images_path", local_server_settings["images_path"]),
            ("projects_path", local_server_settings["projects_path"]),
            ("configs_path", local_server_settings["configs_path"]),
            ("console_start_port_range", local_server_settings["console_start_port_range"]),
            ("console_end_port_range", local_server_settings["console_end_port_range"]),
            ("udp_start_port_range", local_server_settings["udp_start_port_range"]),
            ("udp_start_end_range", local_server_settings["udp_end_port_range"]),
            ("report_errors", local_server_settings["report_errors"]),
        ])
        config = LocalServerConfig.instance()
        config.saveSettings("Server", server_settings)

        if self._local_server and self._local_server.connected():
            self._local_server.post("/config/reload", None)

    def settings(self):
        """
        Returns the servers settings.

        :returns: settings dictionary
        """

        return self._settings

    def setSettings(self, settings):
        """
        Set the servers settings.

        :param settings: settings dictionary
        """

        self._settings.update(settings)

    def localServerSettings(self):
        """
        Returns the local server settings.

        :returns: local server settings (dict)
        """

        return self._settings["local_server"]

    def setLocalServerSettings(self, settings):
        """
        Set new local server settings.

        :param settings: local server settings (dict)
        """

        local_server_settings = self._settings["local_server"]
        local_server_settings.update(settings)

    def vmSettings(self):
        """
        Returns the GNS3 VM settings.

        :returns: GNS3 VM settings (dict)
        """

        return self._settings["vm"]

    def setVMsettings(self, settings):
        """
        Set new GNS3 VM settings.

        :param settings: GNS3 VM settings (dict)
        """

        vm_settings = self._settings["vm"]
        vm_settings.update(settings)

    def shouldLocalServerAutoStart(self):
        """
        Returns either the local server
        is automatically started on startup.

        :returns: boolean
        """

        return self._settings["local_server"]["auto_start"]

    def localServerPath(self):
        """
        Returns the local server path.

        :returns: path to local server program.
        """

        return self._settings["local_server"]["path"]

    def _killAlreadyRunningServer(self):
        """
        Kill a running zombie server (started by a gui that no longer exists)
        This will not kill server started by hand.
        """
        try:
            if os.path.exists(self._pid_path):
                with open(self._pid_path) as f:
                    pid = int(f.read())
                process = psutil.Process(pid=pid)
                log.info("Kill already running server with PID %d", pid)
                process.kill()
        except (OSError, ValueError, psutil.NoSuchProcess, psutil.AccessDenied):
            # Permission issue, or process no longer exists, or file is empty
            return

    def localServerAutoStart(self):
        """
        Try to start the embed gns3 server.
        """

        # We check if two gui are not launched at the same time
        # to avoid killing the server of the other GUI
        if not LocalConfig.isMainGui():
           log.info("Not the main GUI, will not autostart the server")
           return True

        if self.localServer().isLocalServerRunning():
            log.info("A local server already running on this host")
            # Try to kill the server. The server can be still running after
            # if the server was started by hand
            self._killAlreadyRunningServer()

        if not self.localServer().isLocalServerRunning():
            if not self.initLocalServer():
                return False
            if not self.startLocalServer():
                return False
        return True

    def initLocalServer(self):
        """
        Initialize the local server.
        """

        from .main_window import MainWindow
        main_window = MainWindow.instance()

        self._checkUbridgePermissions()

        # check the local server path
        local_server_path = self.localServerPath()
        server = self.localServer()
        if not local_server_path:
            log.warn("No local server is configured")
            return
        if not os.path.isfile(local_server_path):
            QtWidgets.QMessageBox.critical(main_window, "Local server", "Could not find local server {}".format(local_server_path))
            return
        elif not os.access(local_server_path, os.X_OK):
            QtWidgets.QMessageBox.critical(main_window, "Local server", "{} is not an executable".format(local_server_path))
            return

        try:
            # check if the local address still exists
            for res in socket.getaddrinfo(server.host(), 0, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
                af, socktype, proto, _, sa = res
                with socket.socket(af, socktype, proto) as sock:
                    sock.bind(sa)
                    break
        except OSError as e:
            QtWidgets.QMessageBox.critical(main_window, "Local server", "Could not bind with {}: {} (please check your host binding setting in the preferences)".format(server.host(), e))
            return False

        try:
            # check if the port is already taken
            find_unused_port = False
            for res in socket.getaddrinfo(server.host(), server.port(), socket.AF_UNSPEC, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
                af, socktype, proto, _, sa = res
                with socket.socket(af, socktype, proto) as sock:
                    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                    sock.bind(sa)
                    break
        except OSError as e:
            log.warning("Could not use socket {}:{} {}".format(server.host(), server.port(), e))
            find_unused_port = True

        if find_unused_port:
            # find an alternate port for the local server
            old_port = server.port()
            try:
                server.setPort(self._findUnusedLocalPort(server.host()))
            except OSError as e:
                QtWidgets.QMessageBox.critical(main_window, "Local server", "Could not find an unused port for the local server: {}".format(e))
                return False
            log.warning("The server port {} is already in use, fallback to port {}".format(old_port, server.port()))
            print("The server port {} is already in use, fallback to port {}".format(old_port, server.port()))
        return True

    def _findUnusedLocalPort(self, host):
        """
        Find an unused port.

        :param host: server hosts

        :returns: port number
        """

        with socket.socket() as s:
            s.bind((host, 0))
            return s.getsockname()[1]

    def startLocalServer(self):
        """
        Starts the local server process.
        """

        path = self.localServerPath()
        host = self._local_server.host()
        port = self._local_server.port()
        command = '"{executable}" --host={host} --port={port} --local'.format(executable=path,
                                                                              host=host,
                                                                              port=port)

        if self._settings["local_server"]["allow_console_from_anywhere"]:
            # allow connections to console from remote addresses
            command += " --allow"

        if logging.getLogger().isEnabledFor(logging.DEBUG):
            command += " --debug"

        settings_dir = LocalConfig.configDirectory()
        if os.path.isdir(settings_dir):
            # save server logging info to a file in the settings directory
            logpath = os.path.join(settings_dir, "gns3_server.log")
            if os.path.isfile(logpath):
                # delete the previous log file
                try:
                    os.remove(logpath)
                except FileNotFoundError:
                    pass
                except OSError as e:
                    log.warn("could not delete server log file {}: {}".format(logpath, e))
            command += ' --log="{}" --pid="{}"'.format(logpath, self._pid_path)

        log.info("Starting local server process with {}".format(command))
        try:
            if sys.platform.startswith("win"):
                # use the string on Windows
                self._local_server_process = subprocess.Popen(command, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
            else:
                # use arguments on other platforms
                args = shlex.split(command)
                self._local_server_process = subprocess.Popen(args)
        except (OSError, subprocess.SubprocessError) as e:
            log.warning('Could not start local server "{}": {}'.format(command, e))
            return False

        log.info("Local server process has started (PID={})".format(self._local_server_process.pid))
        return True

    def localServerIsRunning(self):
        """
        Returns either the local server is running.

        :returns: boolean
        """

        try:
            if self._local_server_process and self._local_server_process.poll() is None:
                return True
        except OSError:
            pass
        return False

    def stopLocalServer(self, wait=False):
        """
        Stops the local server.

        :param wait: wait for the server to stop
        """

        if self.localServerIsRunning():
            log.info("Stopping local server (PID={})".format(self._local_server_process.pid))
            # local server is running, let's stop it
            if wait:
                try:
                    # wait for the server to stop for maximum 2 seconds
                    self._local_server_process.wait(timeout=2)
                except subprocess.TimeoutExpired:
                    # the local server couldn't be stopped with the normal procedure
                    try:
                        if sys.platform.startswith("win"):
                            self._local_server_process.send_signal(signal.CTRL_BREAK_EVENT)
                        else:
                            self._local_server_process.send_signal(signal.SIGINT)
                    # If the process is already dead we received a permission error
                    # it's a race condition between the timeout and send signal
                    except PermissionError:
                        pass
                    try:
                        # wait for the server to stop for maximum 2 seconds
                        self._local_server_process.wait(timeout=2)
                    except subprocess.TimeoutExpired:
                        from .main_window import MainWindow
                        main_window = MainWindow.instance()
                        proceed = QtWidgets.QMessageBox.question(main_window,
                                                                 "Local server",
                                                                 "The Local server cannot be stopped, would you like to kill it?",
                                                                 QtWidgets.QMessageBox.Yes,
                                                                 QtWidgets.QMessageBox.No)

                        if proceed == QtWidgets.QMessageBox.Yes:
                            self._local_server_process.kill()

    def localServer(self):
        """
        Returns the local server.

        :returns: Server instance
        """

        return self._local_server

    def initVMServer(self):
        """
        Initialize the GNS3 VM server.
        """

        gns3_vm_settings = self._settings["vm"]
        server_info = {
            "host": "unset",
            "port": gns3_vm_settings["server_port"],
            "protocol": "http",
            "user": gns3_vm_settings["user"],
            "password": gns3_vm_settings["password"]
        }
        server = self.getNetworkClientInstance(server_info, self._network_manager)
        server.setLocal(False)
        server.setGNS3VM(True)
        self._vm_server = server
        self.server_added_signal.emit("vm")
        log.info("GNS3 VM server initialized {}".format(server.url()))

    def vmServer(self):
        """
        Returns the GNS3 VM server.

        :returns: Server instance
        """

        return self._vm_server

    def _addRemoteServer(self, protocol="http", host="localhost", port=8000, ram_limit=0, user=None, password=None, accept_insecure_certificate=False, id=None):
        """
        Adds a new remote server.

        :param protocol: Server protocol
        :param host: host or address of the server
        :param port: port of the server (integer)
        :param ram_limit: maximum RAM to be used (integer)
        :param user: user login or None
        :param password: user password or None
        :param accept_insecure_certificate: Accept invalid SSL certificate

        :returns: the new remote server
        """

        server = {"host": host,
                  "port": port,
                  "ram_limit": ram_limit,
                  "protocol": protocol,
                  "user": user,
                  "password": password}
        if accept_insecure_certificate:
            server["accept_insecure_certificate"] = accept_insecure_certificate
        server = self.getNetworkClientInstance(server, self._network_manager)
        server.setLocal(False)
        self._remote_servers[server.url()] = server
        self.server_added_signal.emit(server.url())
        log.info("New remote server connection {} registered".format(server.url()))

        return server

    def getNetworkClientInstance(self, settings, network_manager):
        """
        Based on url return a network client instance
        """

        from gns3.http_client import HTTPClient
        client = HTTPClient(settings, network_manager)
        return client

    def getRemoteServer(self, protocol, host, port, user, settings={}):
        """
        Gets a remote server.

        :param protocol: server protocol (http/https)
        :param host: host address
        :param port: port
        :param user: the username
        :param settings: Additional settings

        :returns: remote server (HTTPClient instance)
        """

        url = getNetworkUrl(protocol, host, port, user, settings)
        for server in self._remote_servers.values():
            if server.url() == url:
                return server

        settings['user'] = user
        settings['protocol'] = protocol
        settings['host'] = host
        settings['port'] = port
        return self._addRemoteServer(**settings)

    def getServerFromString(self, server_name):
        """
        Finds a server instance from its string representation.
        """

        if server_name == "local":
            return self._local_server
        elif server_name == "vm":
            return self._vm_server
        elif server_name == "load-balance":
            return self.anyRemoteServer()

        if "://" in server_name:
            for server in self.servers():
                if server.url() == server_name:
                    return server
            url_settings = urllib.parse.urlparse(server_name)
            settings = {}
            port = url_settings.port
            return self.getRemoteServer(url_settings.scheme, url_settings.hostname, port, url_settings.username, settings=settings)
        else:
            (host, port) = server_name.split(":")
            return self.getRemoteServer("http", host, port, None)

    def anyRemoteServer(self, ram=0):
        """
        Returns a remote server for load balancing.

        :param ram: RAM amount to be allocated by the node

        :returns: remote server (HTTPClient instance)
        """

        if self._settings["load_balancing_method"] == "ram_usage":
            for server in self._remote_servers.values():
                if not server.RAMLimit():
                    return server
                if (server.allocatedRAM() + ram) <= server.RAMLimit():
                    if ram > 0:
                        server.increaseAllocatedRAM(ram)
                    return server
        elif self._settings["load_balancing_method"] == "round_robin":
            return next(iter(self))
        return next(iter(self))  # default is Round-Robin

    def updateRemoteServers(self, servers):
        """
        Updates the remote servers list.

        :param servers: servers dictionary.
        """

        for server_id, server in self._remote_servers.copy().items():
            if server_id not in servers:
                if server.connected():
                    server.close()
                log.info("Remote server connection {} unregistered".format(server.url()))
                del self._remote_servers[server_id]

        for server_id, server in servers.items():
            if server_id in self._remote_servers:
                continue

            new_server = self.getNetworkClientInstance(server, self._network_manager)
            new_server.setLocal(False)
            self._remote_servers[server_id] = new_server
            self.server_added_signal.emit(new_server.url())
            log.info("New remote server connection {} registered".format(new_server.url()))

    def remoteServers(self):
        """
        Returns all the remote servers.

        :returns: remote servers dictionary
        """

        return self._remote_servers

    def __iter__(self):
        """
        Creates a round-robin system to pick up a remote server.
        """

        return self

    def __next__(self):
        """
        Returns the next available remote server.

        :returns: remote server (HTTPClient instance)
        """

        if not self._remote_servers or len(self._remote_servers) == 0:
            return None

        server_ids = list(self._remote_servers.keys())
        try:
            server_id = server_ids[self._remote_server_iter_pos]
        except IndexError:
            self._remote_server_iter_pos = 0
            server_id = server_ids[0]

        if self._remote_server_iter_pos < len(server_ids) - 1:
            self._remote_server_iter_pos += 1
        else:
            self._remote_server_iter_pos = 0

        return self._remote_servers[server_id]

    def save(self):
        """
        Saves the settings.
        """

        self._saveSettings()

    def disconnectAllServers(self):
        """
        Disconnects all servers (local and remote).
        """

        if self._local_server.connected():
            self._local_server.close()
        if self._vm_server is not None and self._vm_server.connected():
            self._vm_server.close()
        for server in self._remote_servers.values():
            if server.connected():
                server.close()

    def isNonLocalServerConfigured(self):
        """
        :returns: True if GNS3 VM or a remote server is configured
        """
        if self._vm_server is not None:
            return True
        if len(self._remote_servers) > 0:
            return True
        return False

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of Servers.

        :returns: instance of Servers
        """

        if not hasattr(Servers, "_instance") or Servers._instance is None:
            Servers._instance = Servers()
        return Servers._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for IOU preferences.
"""

import os
import sys
import shutil

from gns3.qt import QtCore, QtWidgets

from .. import IOU
from ..ui.iou_preferences_page_ui import Ui_IOUPreferencesPageWidget
from ..settings import IOU_SETTINGS


class IOUPreferencesPage(QtWidgets.QWidget, Ui_IOUPreferencesPageWidget):

    """
    QWidget preference page for IOU.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiIOURCPathToolButton.clicked.connect(self._iourcPathBrowserSlot)
        self.uiIouyapPathToolButton.clicked.connect(self._iouyapPathBrowserSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiUseLocalServercheckBox.stateChanged.connect(self._useLocalServerSlot)

        if not sys.platform.startswith("linux"):
            self.uiUseLocalServercheckBox.setChecked(False)
            self.uiUseLocalServercheckBox.setEnabled(False)

    def _iourcPathBrowserSlot(self):
        """
        Slot to open a file browser and select an iourc file
        """

        documents_path = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DocumentsLocation)
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select the IOURC file", documents_path)
        if not path:
            return

        if not os.access(path, os.R_OK):
            QtWidgets.QMessageBox.critical(self, "IOURC file", "{} cannot be read".format(os.path.basename(path)))
            return

        self.uiIOURCPathLineEdit.setText(os.path.normpath(path))

    def _iouyapPathBrowserSlot(self):
        """
        Slot to open a file browser and select iouyap.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"

        iouyap_path = shutil.which("iouyap")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select iouyap", iouyap_path, filter)
        if not path:
            return

        if self._checkIouyapPath(path):
            self.uiIouyapPathLineEdit.setText(os.path.normpath(path))

    def _checkIouyapPath(self, path):
        """
        Checks that the iouyap path is valid.

        :param path: iouyap path
        :returns: boolean
        """

        if not os.path.exists(path):
            QtWidgets.QMessageBox.critical(self, "iouyap", '"{}" does not exist'.format(path))
            return False

        if not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.critical(self, "iouyap", "{} is not an executable".format(os.path.basename(path)))
            return False

        return True

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(IOU_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiIouyapPathLineEdit.setEnabled(True)
            self.uiIouyapPathToolButton.setEnabled(True)
            self.uiLicensecheckBox.setEnabled(True)
        else:
            self.uiIouyapPathLineEdit.setEnabled(False)
            self.uiIouyapPathToolButton.setEnabled(False)
            self.uiLicensecheckBox.setEnabled(False)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: IOU settings
        """

        self.uiIOURCPathLineEdit.setText(settings["iourc_path"])
        self.uiIouyapPathLineEdit.setText(settings["iouyap_path"])
        self.uiLicensecheckBox.setChecked(settings["license_check"])
        self.uiUseLocalServercheckBox.setChecked(settings["use_local_server"])

    def loadPreferences(self):
        """
        Loads IOU preferences.
        """

        iou_settings = IOU.instance().settings()
        self._populateWidgets(iou_settings)

    def savePreferences(self):
        """
        Saves IOU preferences.
        """

        iouyap_path = self.uiIouyapPathLineEdit.text().strip()
        if iouyap_path and self.uiUseLocalServercheckBox.isChecked() and not self._checkIouyapPath(iouyap_path):
            return

        iourc_path = self.uiIOURCPathLineEdit.text().strip()
        if iourc_path and self.uiUseLocalServercheckBox.isChecked() and iourc_path and not os.path.exists(iourc_path):
            QtWidgets.QMessageBox.critical(self, "iourc", '"{}" does not exist'.format(iourc_path))
            return

        new_settings = {"iouyap_path": iouyap_path,
                        "iourc_path": iourc_path,
                        "license_check": self.uiLicensecheckBox.isChecked(),
                        "use_local_server": self.uiUseLocalServercheckBox.isChecked()}
        IOU.instance().setSettings(new_settings)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for Device classes.
"""


from gns3.node import Node
from gns3.packet_capture import PacketCapture

import logging
log = logging.getLogger(__name__)


class Device(Node):

    URL_PREFIX = "dynamips"

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        self._device_id = None

    def device_id(self):
        """
        Return the ID of this device

        :returns: identifier (string)
        """

        return self._device_id

    def _setupCallback(self, result, error=False, **kwargs):
        """
        Callback for setup.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while setting up {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return

        self._device_id = result["device_id"]
        self._settings["name"] = result["name"]
        log.info("{} has been created".format(self.name()))
        self.setInitialized(True)
        self.created_signal.emit(self.id())
        self._module.addNode(self)

    def delete(self):
        """
        Deletes this Device instance.
        """

        log.debug("{} is being deleted".format(self.name()))
        # first delete all the links attached to this node
        self.delete_links_signal.emit()
        if self._device_id and self._server.connected():
            self.httpDelete("/{prefix}/devices/{device_id}".format(prefix=self.URL_PREFIX, device_id=self._device_id),
                            self._deleteCallback)
        else:
            self.deleted_signal.emit()
            self._module.removeNode(self)

    def _deleteCallback(self, result, error=False, **kwargs):
        """
        Callback for delete.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        log.info("{} has been deleted".format(self.name()))
        self.deleted_signal.emit()
        self._module.removeNode(self)

    def _addNIOCallback(self, result, error=False, context={}, **kwargs):
        """
        Callback for addNIO.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while adding a NIO for {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            self.nio_cancel_signal.emit(self.id())
        else:
            self.nio_signal.emit(self.id(), context["port_id"])

    def deleteNIO(self, port):
        """
        Deletes an NIO from the specified port on this Device instance

        :param port: Port instance
        """

        log.debug("{} is deleting an NIO".format(self.name()))
        self.httpDelete("/{prefix}/devices/{device_id}/ports/{port}/nio".format(prefix=self.URL_PREFIX,
                                                                                port=port.portNumber(),
                                                                                device_id=self._device_id),
                        self._deleteNIOCallback)

    def _deleteNIOCallback(self, result, error=False, **kwargs):
        """
        Callback for deleteNIO.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("Error while deleting NIO {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return

        log.debug("{} has deleted a NIO: {}".format(self.name(), result))

    def startPacketCapture(self, port, capture_file_name, data_link_type):
        """
        Starts a packet capture.

        :param port: Port instance
        :param capture_file_name: PCAP capture file path
        :param data_link_type: PCAP data link type
        """

        params = {"capture_file_name": capture_file_name,
                  "data_link_type": data_link_type}

        log.debug("{} is starting a packet capture on {}: {}".format(self.name(), port.name(), params))
        self.httpPost("/{prefix}/devices/{device_id}/ports/{port}/start_capture".format(
            port=port.portNumber(),
            prefix=self.URL_PREFIX,
            device_id=self._device_id),
            self._startPacketCaptureCallback,
            context={"port": port},
            body=params)

    def _startPacketCaptureCallback(self, result, error=False, context={}, **kwargs):
        """
        Callback for starting a packet capture.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while starting capture {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            PacketCapture.instance().startCapture(self, context["port"], result["pcap_file_path"])

    def stopPacketCapture(self, port):
        """
        Stops a packet capture.

        :param port: Port instance
        """

        log.debug("{} is stopping a packet capture on {}".format(self.name(), port.name()))
        self.httpPost("/{prefix}/devices/{device_id}/ports/{port}/stop_capture".format(
            port=port.portNumber(),
            prefix=self.URL_PREFIX,
            device_id=self._device_id),
            self._stopPacketCaptureCallback,
            context={"port": port})

    def _stopPacketCaptureCallback(self, result, error=False, context={}, **kwargs):
        """
        Callback for stopping a packet capture.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while stopping capture {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            PacketCapture.instance().stopCapture(self, context["port"])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Manages the GNS3 VM.
"""

import os
import sys
import subprocess
import codecs
import shutil

from .qt import QtNetwork
from collections import OrderedDict
from .servers import Servers

import logging
log = logging.getLogger(__name__)


class GNS3VM:

    """
    GNS3 VM management class.
    """

    def __init__(self):

        self._is_running = False
        # The current running vboxmanage and vmrun process
        self._running_process = None

    def settings(self):
        """
        Returns the GNS3 VM settings.

        :returns: GNS3 VM settings (dict)
        """

        return Servers.instance().vmSettings()

    def setSettings(self, settings):
        """
        Set new GNS3 VM settings.

        :param settings: GNS3 VM settings (dict)
        """

        Servers.instance().setVMsettings(settings)

    def killRunningProcess(self):
        """
        Kill the VBoxManage or vmrun process if running
        """
        if self._running_process is not None:
            self._running_process.kill()
            self._running_process.wait()
            self._running_process = None

    def _process_check_output(self, command, timeout=None):
        # Original code from Python's subprocess.check_output
        # https://github.com/python/cpython/blob/3.4/Lib/subprocess.py
        with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=os.environ) as process:
            self._running_process = process
            try:
                output, unused_err = process.communicate(None, timeout=timeout)
            except subprocess.TimeoutExpired:
                process.kill()
                output, unused_err = process.communicate()
                self._running_process = None
                raise subprocess.TimeoutExpired(process.args, timeout, output=output)
            except:
                self.killRunningProcess()
                raise
            retcode = process.poll()
            if retcode:
                self._running_process = None
                raise subprocess.CalledProcessError(retcode, process.args, output=output)
        self._running_process = None
        return output.decode("utf-8", errors="ignore").strip()

    def execute_vmrun(self, subcommand, args, timeout=60):

        from gns3.modules.vmware import VMware
        vmware_settings = VMware.instance().settings()
        vmrun_path = vmware_settings["vmrun_path"]
        if sys.platform.startswith("darwin"):
            command = [vmrun_path, "-T", "fusion", subcommand]
        else:
            host_type = vmware_settings["host_type"]
            command = [vmrun_path, "-T", host_type, subcommand]
        command.extend(args)
        log.debug("Executing vmrun with command: {}".format(command))
        return self._process_check_output(command, timeout=timeout)

    def execute_vboxmanage(self, subcommand, args, timeout=60):

        from gns3.modules.virtualbox import VirtualBox
        virtualbox_settings = VirtualBox.instance().settings()
        vboxmanage_path = virtualbox_settings["vboxmanage_path"]
        command = [vboxmanage_path, "--nologo", subcommand]
        command.extend(args)
        log.debug("Executing VBoxManage with command: {}".format(command))
        return self._process_check_output(command, timeout=timeout)

    @staticmethod
    def parse_vmx_file(path):
        """
        Parses a VMX file.

        :param path: path to the VMX file

        :returns: dict
        """

        pairs = OrderedDict()
        encoding = "utf-8"
        # get the first line to read the .encoding value
        with open(path, "rb") as f:
            line = f.readline().decode(encoding, errors="ignore")
            if line.startswith("#!"):
                # skip the shebang
                line = f.readline().decode(encoding, errors="ignore")
            try:
                key, value = line.split('=', 1)
                if key.strip().lower() == ".encoding":
                    file_encoding = value.strip('" ')
                    try:
                        codecs.lookup(file_encoding)
                        encoding = file_encoding
                    except LookupError:
                        log.warning("Invalid file encoding detected in '{}': {}".format(path, file_encoding))
            except ValueError:
                log.warning("Couldn't find file encoding in {}, using {}...".format(path, encoding))

        # read the file with the correct encoding
        with open(path, encoding=encoding, errors="ignore") as f:
            for line in f.read().splitlines():
                try:
                    key, value = line.split('=', 1)
                    pairs[key.strip().lower()] = value.strip('" ')
                except ValueError:
                    continue
        return pairs

    @staticmethod
    def write_vmx_file(path, pairs):
        """
        Write a VMware VMX file.

        :param path: path to the VMX file
        :param pairs: settings to write
        """

        encoding = "utf-8"
        if ".encoding" in pairs:
            file_encoding = pairs[".encoding"]
            try:
                codecs.lookup(file_encoding)
                encoding = file_encoding
            except LookupError:
                log.warning("Invalid file encoding detected in '{}': {}".format(path, file_encoding))
        with open(path, "w", encoding=encoding, errors="ignore") as f:
            if sys.platform.startswith("linux"):
                # write the shebang on the first line on Linux
                vmware_path = shutil.which("vmware")
                if vmware_path:
                    f.write("#!{}\n".format(vmware_path))
            for key, value in pairs.items():
                entry = '{} = "{}"\n'.format(key, value)
                f.write(entry)

    def autoStart(self):
        """
        Automatically start the GNS3 VM at startup.

        :returns: boolean
        """

        vm_settings = Servers.instance().vmSettings()
        return vm_settings["auto_start"]

    def adjustLocalServerIP(self):
        """
        Adjust the local server IP address to be in the same subnet as the GNS3 VM.

        :returns: the local server IP/host address
        """

        servers = Servers.instance()
        local_server_settings = servers.localServerSettings()
        if Servers.instance().vmSettings()["adjust_local_server_ip"]:
            vm_server = servers.vmServer()
            vm_ip_address = vm_server.host()
            log.debug("GNS3 VM IP address is {}".format(vm_ip_address))

            for interface in QtNetwork.QNetworkInterface.allInterfaces():
                for address in interface.addressEntries():
                    ip = address.ip().toString()
                    prefix_length = address.prefixLength()
                    subnet = QtNetwork.QHostAddress.parseSubnet("{}/{}".format(ip, prefix_length))
                    if QtNetwork.QHostAddress(vm_ip_address).isInSubnet(subnet):
                        if local_server_settings["host"] != ip:
                            log.info("Adjust local server IP address to {}".format(ip))
                            servers.setLocalServerSettings({"host": ip})
                            servers.registerLocalServer()
                            servers.save()
                            return ip
        return local_server_settings["host"]

    def setRunning(self, value):
        """
        Sets either the GNS3 VM is running or not.

        :param value: boolean
        """

        self._is_running = value

    def isRunning(self):
        """
        Returns either the GNS3 VM is running or not.

        :returns: boolean
        """

        return self._is_running

    def setvCPUandRAM(self, vcpus, ram):
        """
        Set the vCPU cores and RAM amount for the GNS3 VM.

        :param vcpus: number of vCPU cores
        :param ram: amount of memory

        :returns: boolean
        """

        vm_settings = self.settings()
        if vm_settings["virtualization"] == "VMware":
            try:
                pairs = self.parse_vmx_file(vm_settings["vmx_path"])
                pairs["numvcpus"] = str(vcpus)
                pairs["memsize"] = str(ram)
                self.write_vmx_file(vm_settings["vmx_path"], pairs)
            except OSError as e:
                log.error('Could not read/write VMware VMX file "{}": {}'.format(vm_settings["vmx_path"], e))
                return False

        elif vm_settings["virtualization"] == "VirtualBox":
            try:
                self.execute_vboxmanage("modifyvm", [vm_settings["vmname"], "--cpus", str(vcpus)], timeout=3)
                self.execute_vboxmanage("modifyvm", [vm_settings["vmname"], "--memory", str(ram)], timeout=3)
            except OSError as e:
                log.error("Could not execute VBoxManage: {}".format(e), True)
                return False
            except subprocess.SubprocessError as e:
                log.error("Could not execute VBoxManage: {} with output '{}'".format(e, e.output.decode("utf-8", errors="ignore").strip()), True)
                return False
            except subprocess.TimeoutExpired:
                log.error("VBoxmanage timeout expired", True)
                return False
        log.info("GNS3 VM vCPU count set to {} and RAM to {} MB".format(vcpus, ram))
        return True

    def shutdown(self, force=False):
        """
        Gracefully shutdowns the GNS3 VM.
        """

        vm_settings = self.settings()
        if self._is_running and (vm_settings["auto_stop"] or force):
            try:
                if vm_settings["virtualization"] == "VMware":
                    if vm_settings["vmx_path"] is None:
                        log.error("No vm path configured, can't stop the VM")
                        return
                    self.execute_vmrun("stop", [vm_settings["vmx_path"], "soft"])
                elif vm_settings["virtualization"] == "VirtualBox":
                    self.execute_vboxmanage("controlvm", [vm_settings["vmname"], "acpipowerbutton"], timeout=3)
            except (OSError, subprocess.SubprocessError):
                pass
            except subprocess.TimeoutExpired:
                log.warning("Could not ACPI shutdown the VM (timeout expired)")
            self._is_running = False

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of GNS3VM

        :returns: instance of GNS3VM
        """

        if not hasattr(GNS3VM, "_instance") or GNS3VM._instance is None:
            GNS3VM._instance = GNS3VM()
        return GNS3VM._instance



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/preferences_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PreferencesDialog(object):
    def setupUi(self, PreferencesDialog):
        PreferencesDialog.setObjectName("PreferencesDialog")
        PreferencesDialog.resize(900, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(PreferencesDialog.sizePolicy().hasHeightForWidth())
        PreferencesDialog.setSizePolicy(sizePolicy)
        PreferencesDialog.setMaximumSize(QtCore.QSize(900, 600))
        PreferencesDialog.setSizeGripEnabled(False)
        PreferencesDialog.setModal(True)
        self.horizontalLayout = QtWidgets.QHBoxLayout(PreferencesDialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiLine = QtWidgets.QFrame(PreferencesDialog)
        self.uiLine.setFrameShape(QtWidgets.QFrame.HLine)
        self.uiLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.uiLine.setObjectName("uiLine")
        self.horizontalLayout.addWidget(self.uiLine)
        self.uiTreeWidget = QtWidgets.QTreeWidget(PreferencesDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiTreeWidget.setSizePolicy(sizePolicy)
        self.uiTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiTreeWidget.setFont(font)
        self.uiTreeWidget.setIndentation(10)
        self.uiTreeWidget.setObjectName("uiTreeWidget")
        self.uiTreeWidget.headerItem().setText(0, "1")
        self.uiTreeWidget.header().setVisible(False)
        self.horizontalLayout.addWidget(self.uiTreeWidget)
        self.vbox = QtWidgets.QVBoxLayout()
        self.vbox.setSpacing(3)
        self.vbox.setObjectName("vbox")
        self.uiTitleLabel = QtWidgets.QLabel(PreferencesDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTitleLabel.sizePolicy().hasHeightForWidth())
        self.uiTitleLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setUnderline(False)
        font.setWeight(75)
        self.uiTitleLabel.setFont(font)
        self.uiTitleLabel.setFrameShape(QtWidgets.QFrame.Box)
        self.uiTitleLabel.setObjectName("uiTitleLabel")
        self.vbox.addWidget(self.uiTitleLabel)
        self.scrollArea = QtWidgets.QScrollArea(PreferencesDialog)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 680, 519))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiStackedWidget = QtWidgets.QStackedWidget(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.uiStackedWidget.sizePolicy().hasHeightForWidth())
        self.uiStackedWidget.setSizePolicy(sizePolicy)
        self.uiStackedWidget.setObjectName("uiStackedWidget")
        self.uiPageWidget = QtWidgets.QWidget()
        self.uiPageWidget.setObjectName("uiPageWidget")
        self.uiStackedWidget.addWidget(self.uiPageWidget)
        self.verticalLayout.addWidget(self.uiStackedWidget)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)
        self.vbox.addWidget(self.scrollArea)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(PreferencesDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Apply|QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setCenterButtons(False)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.vbox.addWidget(self.uiButtonBox)
        self.horizontalLayout.addLayout(self.vbox)

        self.retranslateUi(PreferencesDialog)
        self.uiButtonBox.accepted.connect(PreferencesDialog.accept)
        self.uiButtonBox.rejected.connect(PreferencesDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(PreferencesDialog)

    def retranslateUi(self, PreferencesDialog):
        _translate = QtCore.QCoreApplication.translate
        PreferencesDialog.setWindowTitle(_translate("PreferencesDialog", "Preferences"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from ..qt import QtCore
from ..utils import md5_hash_file

import logging
log = logging.getLogger(__name__)


class DownloadProjectWorker(QtCore.QObject):

    """
    Downloads project from cloud storage
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)
    file_downloaded = QtCore.pyqtSignal()
    file_list_received = QtCore.pyqtSignal()

    def __init__(self, parent, project, servers):
        self._is_running = False
        self._project = project
        self._servers = servers
        self._files_to_download = []
        self._get_file_lists = 0  # Counter to know how many file list remain to download
        self._total_files_to_download = 0
        super().__init__(parent)

    def run(self):
        self.updated.emit(0)
        self._is_running = True

        try:
            if self._servers.vmServer():
                self._project.get(self._servers.vmServer(), "/files", self._fileListReceived)
                self._get_file_lists += 1
            for server in self._servers.remoteServers().values():
                self._project.get(server, "/files", self._fileListReceived)
                self._get_file_lists += 1
        except Exception as e:
            self.error.emit("Error importing project: {}".format(e), True)

        if self._get_file_lists == 0:
            self._is_running = False

    def _fileListReceived(self, result, error=False, server=None, **kwargs):
        self._get_file_lists -= 1
        if error:
            msg = "Error while downloading project {}".format(result["message"])
            log.error(msg)
            self.error.emit(msg, True)
            return

        for file in result:
            file["server"] = server
            self._files_to_download.append(file)
            self._total_files_to_download += 1

        if self._get_file_lists <= 0:
            self._downloadNextFile()
        else:
            self._is_running = False
            self.finished.emit()

    def _downloadNextFile(self):
        if not self._is_running:
            return

        try:
            file_to_download = self._files_to_download.pop()
        except IndexError:
            self.finished.emit()
            return

        file_path = os.path.join(self._project.filesDir(), file_to_download["path"])

        self.updated.emit(round(100.0 / self._total_files_to_download * (self._total_files_to_download - (len(self._files_to_download) + 1))))

        try:
            if os.path.dirname(file_path) is not None:
                os.makedirs(os.path.dirname(file_path), exist_ok=True)

            if os.path.exists(file_path):
                if md5_hash_file(file_path) == file_to_download["md5sum"]:
                    self._downloadNextFile()
                    return

            f = open(file_path, "wb+")
        except OSError as e:
            self.error.emit("Could not write file {}: {}".format(file_path, e), False)
            return
        self._project.get(file_to_download["server"], "/files/{}".format(file_to_download["path"]), self._downloadFileReceived, context={"fd": f, "file_path": file_path}, downloadProgressCallback=self._downloadFileProgress)

    def _downloadFileReceived(self, content, error=False, server=None, context={}):
        """
        Called when download finish
        """
        context["fd"].close()
        self._downloadNextFile()

    def _downloadFileProgress(self, content, server=None, context={}):
        """
        Called for each part of the file
        """
        try:
            context["fd"].write(content)
        except OSError as e:
            self.error.emit("Could not write file {}: {}".format(context["file_path"], e), False)

    def cancel(self):
        """
        Cancel this worker.
        """

        if not self:
            return
        self._is_running = False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/main_window.ui'
#
# Created: Tue Apr 26 11:38:43 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.resize(984, 715)
        MainWindow.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        MainWindow.setStyleSheet("#toolBar_Devices QToolButton {\n"
"width: 50px;\n"
"height: 55px;\n"
"border:solid 1px black opacity 0.4;\n"
"background-none;\n"
"}\n"
"\n"
"#toolBar_General QToolButton {\n"
"width: 36px;\n"
"height: 36px;\n"
"border:solid 1px black opacity 0.4;\n"
"background-none;\n"
"}\n"
"\n"
"")
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AllowTabbedDocks|QtWidgets.QMainWindow.AnimatedDocks)
        self.uiCentralWidget = QtWidgets.QWidget(MainWindow)
        self.uiCentralWidget.setObjectName("uiCentralWidget")
        self.gridlayout = QtWidgets.QGridLayout(self.uiCentralWidget)
        self.gridlayout.setContentsMargins(0, 0, 0, 0)
        self.gridlayout.setSpacing(0)
        self.gridlayout.setObjectName("gridlayout")
        self.uiGraphicsView = GraphicsView(self.uiCentralWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiGraphicsView.sizePolicy().hasHeightForWidth())
        self.uiGraphicsView.setSizePolicy(sizePolicy)
        self.uiGraphicsView.setObjectName("uiGraphicsView")
        self.gridlayout.addWidget(self.uiGraphicsView, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.uiCentralWidget)
        self.uiMenuBar = QtWidgets.QMenuBar(MainWindow)
        self.uiMenuBar.setGeometry(QtCore.QRect(0, 0, 984, 25))
        self.uiMenuBar.setObjectName("uiMenuBar")
        self.uiEditMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiEditMenu.setObjectName("uiEditMenu")
        self.uiFileMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiFileMenu.setObjectName("uiFileMenu")
        self.uiHelpMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiHelpMenu.setObjectName("uiHelpMenu")
        self.uiViewMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiViewMenu.setObjectName("uiViewMenu")
        self.uiDocksMenu = QtWidgets.QMenu(self.uiViewMenu)
        self.uiDocksMenu.setObjectName("uiDocksMenu")
        self.uiControlMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiControlMenu.setObjectName("uiControlMenu")
        self.uiAnnotateMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiAnnotateMenu.setObjectName("uiAnnotateMenu")
        self.uiDeviceMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiDeviceMenu.setObjectName("uiDeviceMenu")
        self.uiToolsMenu = QtWidgets.QMenu(self.uiMenuBar)
        self.uiToolsMenu.setObjectName("uiToolsMenu")
        MainWindow.setMenuBar(self.uiMenuBar)
        self.uiStatusBar = QtWidgets.QStatusBar(MainWindow)
        self.uiStatusBar.setObjectName("uiStatusBar")
        MainWindow.setStatusBar(self.uiStatusBar)
        self.uiGeneralToolBar = QtWidgets.QToolBar(MainWindow)
        self.uiGeneralToolBar.setToolTip("")
        self.uiGeneralToolBar.setStatusTip("")
        self.uiGeneralToolBar.setOrientation(QtCore.Qt.Horizontal)
        self.uiGeneralToolBar.setIconSize(QtCore.QSize(32, 32))
        self.uiGeneralToolBar.setObjectName("uiGeneralToolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.uiGeneralToolBar)
        self.uiNodesDockWidget = QtWidgets.QDockWidget(MainWindow)
        self.uiNodesDockWidget.setEnabled(True)
        self.uiNodesDockWidget.setVisible(True)
        self.uiNodesDockWidget.setFloating(False)
        self.uiNodesDockWidget.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.uiNodesDockWidget.setObjectName("uiNodesDockWidget")
        self.uiNodesDockWidgetContents = QtWidgets.QWidget()
        self.uiNodesDockWidgetContents.setObjectName("uiNodesDockWidgetContents")
        self.vboxlayout = QtWidgets.QVBoxLayout(self.uiNodesDockWidgetContents)
        self.vboxlayout.setSpacing(0)
        self.vboxlayout.setContentsMargins(0, 0, 0, 0)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiNodesView = NodesView(self.uiNodesDockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNodesView.sizePolicy().hasHeightForWidth())
        self.uiNodesView.setSizePolicy(sizePolicy)
        self.uiNodesView.setDragEnabled(False)
        self.uiNodesView.setIconSize(QtCore.QSize(32, 32))
        self.uiNodesView.setRootIsDecorated(False)
        self.uiNodesView.setObjectName("uiNodesView")
        self.uiNodesView.header().setVisible(False)
        self.vboxlayout.addWidget(self.uiNodesView)
        self.uiNodesDockWidget.setWidget(self.uiNodesDockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.uiNodesDockWidget)
        self.uiBrowsersToolBar = QtWidgets.QToolBar(MainWindow)
        self.uiBrowsersToolBar.setIconSize(QtCore.QSize(48, 48))
        self.uiBrowsersToolBar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.uiBrowsersToolBar.setObjectName("uiBrowsersToolBar")
        MainWindow.addToolBar(QtCore.Qt.LeftToolBarArea, self.uiBrowsersToolBar)
        self.uiControlToolBar = QtWidgets.QToolBar(MainWindow)
        self.uiControlToolBar.setIconSize(QtCore.QSize(32, 32))
        self.uiControlToolBar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.uiControlToolBar.setObjectName("uiControlToolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.uiControlToolBar)
        self.uiConsoleDockWidget = QtWidgets.QDockWidget(MainWindow)
        self.uiConsoleDockWidget.setMaximumSize(QtCore.QSize(524287, 524287))
        self.uiConsoleDockWidget.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)
        self.uiConsoleDockWidget.setObjectName("uiConsoleDockWidget")
        self.uiConsoleDockWidgetContents = QtWidgets.QWidget()
        self.uiConsoleDockWidgetContents.setObjectName("uiConsoleDockWidgetContents")
        self.vboxlayout1 = QtWidgets.QVBoxLayout(self.uiConsoleDockWidgetContents)
        self.vboxlayout1.setSpacing(0)
        self.vboxlayout1.setContentsMargins(0, 0, 0, 0)
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.uiConsoleTextEdit = ConsoleView(self.uiConsoleDockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConsoleTextEdit.sizePolicy().hasHeightForWidth())
        self.uiConsoleTextEdit.setSizePolicy(sizePolicy)
        self.uiConsoleTextEdit.setObjectName("uiConsoleTextEdit")
        self.vboxlayout1.addWidget(self.uiConsoleTextEdit)
        self.uiConsoleDockWidget.setWidget(self.uiConsoleDockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.uiConsoleDockWidget)
        self.uiAnnotationToolBar = QtWidgets.QToolBar(MainWindow)
        self.uiAnnotationToolBar.setIconSize(QtCore.QSize(32, 32))
        self.uiAnnotationToolBar.setObjectName("uiAnnotationToolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.uiAnnotationToolBar)
        self.uiTopologySummaryDockWidget = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTopologySummaryDockWidget.sizePolicy().hasHeightForWidth())
        self.uiTopologySummaryDockWidget.setSizePolicy(sizePolicy)
        self.uiTopologySummaryDockWidget.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)
        self.uiTopologySummaryDockWidget.setObjectName("uiTopologySummaryDockWidget")
        self.uiTopologySummaryDockWidgetContents = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTopologySummaryDockWidgetContents.sizePolicy().hasHeightForWidth())
        self.uiTopologySummaryDockWidgetContents.setSizePolicy(sizePolicy)
        self.uiTopologySummaryDockWidgetContents.setObjectName("uiTopologySummaryDockWidgetContents")
        self.gridlayout1 = QtWidgets.QGridLayout(self.uiTopologySummaryDockWidgetContents)
        self.gridlayout1.setContentsMargins(0, 0, 0, 0)
        self.gridlayout1.setSpacing(0)
        self.gridlayout1.setObjectName("gridlayout1")
        self.uiTopologySummaryTreeWidget = TopologySummaryView(self.uiTopologySummaryDockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTopologySummaryTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiTopologySummaryTreeWidget.setSizePolicy(sizePolicy)
        self.uiTopologySummaryTreeWidget.setObjectName("uiTopologySummaryTreeWidget")
        self.uiTopologySummaryTreeWidget.header().setVisible(False)
        self.gridlayout1.addWidget(self.uiTopologySummaryTreeWidget, 0, 0, 1, 1)
        self.uiTopologySummaryDockWidget.setWidget(self.uiTopologySummaryDockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.uiTopologySummaryDockWidget)
        self.uiServerSummaryDockWidget = QtWidgets.QDockWidget(MainWindow)
        self.uiServerSummaryDockWidget.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)
        self.uiServerSummaryDockWidget.setObjectName("uiServerSummaryDockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.dockWidgetContents)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.uiServerSummaryTreeWidget = ServerSummaryView(self.dockWidgetContents)
        self.uiServerSummaryTreeWidget.setObjectName("uiServerSummaryTreeWidget")
        self.uiServerSummaryTreeWidget.headerItem().setText(0, "1")
        self.uiServerSummaryTreeWidget.header().setVisible(False)
        self.gridLayout.addWidget(self.uiServerSummaryTreeWidget, 0, 0, 1, 1)
        self.uiServerSummaryDockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.uiServerSummaryDockWidget)
        self.uiAboutAction = QtWidgets.QAction(MainWindow)
        self.uiAboutAction.setMenuRole(QtWidgets.QAction.AboutRole)
        self.uiAboutAction.setObjectName("uiAboutAction")
        self.uiQuitAction = QtWidgets.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/quit.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiQuitAction.setIcon(icon)
        self.uiQuitAction.setObjectName("uiQuitAction")
        self.uiOpenProjectAction = QtWidgets.QAction(MainWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/open.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiOpenProjectAction.setIcon(icon1)
        self.uiOpenProjectAction.setObjectName("uiOpenProjectAction")
        self.uiSaveProjectAction = QtWidgets.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/save.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiSaveProjectAction.setIcon(icon2)
        self.uiSaveProjectAction.setObjectName("uiSaveProjectAction")
        self.uiOnlineHelpAction = QtWidgets.QAction(MainWindow)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/help.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiOnlineHelpAction.setIcon(icon3)
        self.uiOnlineHelpAction.setObjectName("uiOnlineHelpAction")
        self.uiScreenshotAction = QtWidgets.QAction(MainWindow)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/camera-photo.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon4.addPixmap(QtGui.QPixmap(":/icons/camera-photo-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiScreenshotAction.setIcon(icon4)
        self.uiScreenshotAction.setObjectName("uiScreenshotAction")
        self.uiStartAllAction = QtWidgets.QAction(MainWindow)
        self.uiStartAllAction.setEnabled(True)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/start.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon5.addPixmap(QtGui.QPixmap(":/icons/start-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiStartAllAction.setIcon(icon5)
        self.uiStartAllAction.setObjectName("uiStartAllAction")
        self.uiStopAllAction = QtWidgets.QAction(MainWindow)
        self.uiStopAllAction.setEnabled(True)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon6.addPixmap(QtGui.QPixmap(":/icons/stop-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiStopAllAction.setIcon(icon6)
        self.uiStopAllAction.setObjectName("uiStopAllAction")
        self.uiConsoleAllAction = QtWidgets.QAction(MainWindow)
        self.uiConsoleAllAction.setEnabled(True)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/console.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiConsoleAllAction.setIcon(icon7)
        self.uiConsoleAllAction.setObjectName("uiConsoleAllAction")
        self.uiAboutQtAction = QtWidgets.QAction(MainWindow)
        self.uiAboutQtAction.setMenuRole(QtWidgets.QAction.AboutQtRole)
        self.uiAboutQtAction.setObjectName("uiAboutQtAction")
        self.uiZoomInAction = QtWidgets.QAction(MainWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/icons/zoom-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon8.addPixmap(QtGui.QPixmap(":/icons/zoom-in-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiZoomInAction.setIcon(icon8)
        self.uiZoomInAction.setObjectName("uiZoomInAction")
        self.uiZoomOutAction = QtWidgets.QAction(MainWindow)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/icons/zoom-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon9.addPixmap(QtGui.QPixmap(":/icons/zoom-out-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiZoomOutAction.setIcon(icon9)
        self.uiZoomOutAction.setObjectName("uiZoomOutAction")
        self.uiZoomResetAction = QtWidgets.QAction(MainWindow)
        self.uiZoomResetAction.setObjectName("uiZoomResetAction")
        self.uiSelectAllAction = QtWidgets.QAction(MainWindow)
        self.uiSelectAllAction.setObjectName("uiSelectAllAction")
        self.uiSelectNoneAction = QtWidgets.QAction(MainWindow)
        self.uiSelectNoneAction.setObjectName("uiSelectNoneAction")
        self.uiPreferencesAction = QtWidgets.QAction(MainWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/icons/applications.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiPreferencesAction.setIcon(icon10)
        self.uiPreferencesAction.setMenuRole(QtWidgets.QAction.PreferencesRole)
        self.uiPreferencesAction.setObjectName("uiPreferencesAction")
        self.uiSuspendAllAction = QtWidgets.QAction(MainWindow)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/icons/pause.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon11.addPixmap(QtGui.QPixmap(":/icons/pause-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiSuspendAllAction.setIcon(icon11)
        self.uiSuspendAllAction.setObjectName("uiSuspendAllAction")
        self.uiAddNoteAction = QtWidgets.QAction(MainWindow)
        self.uiAddNoteAction.setCheckable(True)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/icons/add-note.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiAddNoteAction.setIcon(icon12)
        self.uiAddNoteAction.setObjectName("uiAddNoteAction")
        self.uiNewProjectAction = QtWidgets.QAction(MainWindow)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/icons/new-project.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiNewProjectAction.setIcon(icon13)
        self.uiNewProjectAction.setObjectName("uiNewProjectAction")
        self.uiImportExportConfigsAction = QtWidgets.QAction(MainWindow)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/icons/import_export_configs.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiImportExportConfigsAction.setIcon(icon14)
        self.uiImportExportConfigsAction.setObjectName("uiImportExportConfigsAction")
        self.uiInsertImageAction = QtWidgets.QAction(MainWindow)
        self.uiInsertImageAction.setCheckable(False)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/icons/image.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiInsertImageAction.setIcon(icon15)
        self.uiInsertImageAction.setObjectName("uiInsertImageAction")
        self.uiDrawRectangleAction = QtWidgets.QAction(MainWindow)
        self.uiDrawRectangleAction.setCheckable(True)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/icons/rectangle.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon16.addPixmap(QtGui.QPixmap(":/icons/rectangle-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiDrawRectangleAction.setIcon(icon16)
        self.uiDrawRectangleAction.setObjectName("uiDrawRectangleAction")
        self.uiDrawEllipseAction = QtWidgets.QAction(MainWindow)
        self.uiDrawEllipseAction.setCheckable(True)
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/icons/ellipse.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon17.addPixmap(QtGui.QPixmap(":/icons/ellipse-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiDrawEllipseAction.setIcon(icon17)
        self.uiDrawEllipseAction.setObjectName("uiDrawEllipseAction")
        self.uiShowPortNamesAction = QtWidgets.QAction(MainWindow)
        self.uiShowPortNamesAction.setCheckable(True)
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/icons/show-interface-names.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiShowPortNamesAction.setIcon(icon18)
        self.uiShowPortNamesAction.setObjectName("uiShowPortNamesAction")
        self.uiSnapshotAction = QtWidgets.QAction(MainWindow)
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/icons/snapshot.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiSnapshotAction.setIcon(icon19)
        self.uiSnapshotAction.setObjectName("uiSnapshotAction")
        self.uiShowLayersAction = QtWidgets.QAction(MainWindow)
        self.uiShowLayersAction.setCheckable(True)
        self.uiShowLayersAction.setObjectName("uiShowLayersAction")
        self.uiSaveProjectAsAction = QtWidgets.QAction(MainWindow)
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/icons/save-as-project.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiSaveProjectAsAction.setIcon(icon20)
        self.uiSaveProjectAsAction.setObjectName("uiSaveProjectAsAction")
        self.uiReloadAllAction = QtWidgets.QAction(MainWindow)
        icon21 = QtGui.QIcon()
        icon21.addPixmap(QtGui.QPixmap(":/icons/reload.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiReloadAllAction.setIcon(icon21)
        self.uiReloadAllAction.setObjectName("uiReloadAllAction")
        self.uiAuxConsoleAllAction = QtWidgets.QAction(MainWindow)
        self.uiAuxConsoleAllAction.setEnabled(True)
        icon22 = QtGui.QIcon()
        icon22.addPixmap(QtGui.QPixmap(":/icons/aux-console.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiAuxConsoleAllAction.setIcon(icon22)
        self.uiAuxConsoleAllAction.setObjectName("uiAuxConsoleAllAction")
        self.uiResetPortLabelsAction = QtWidgets.QAction(MainWindow)
        self.uiResetPortLabelsAction.setObjectName("uiResetPortLabelsAction")
        self.uiCheckForUpdateAction = QtWidgets.QAction(MainWindow)
        self.uiCheckForUpdateAction.setObjectName("uiCheckForUpdateAction")
        self.uiEnergySavingStyleAction = QtWidgets.QAction(MainWindow)
        self.uiEnergySavingStyleAction.setCheckable(True)
        self.uiEnergySavingStyleAction.setObjectName("uiEnergySavingStyleAction")
        self.uiDefaultStyleAction = QtWidgets.QAction(MainWindow)
        self.uiDefaultStyleAction.setCheckable(True)
        self.uiDefaultStyleAction.setChecked(True)
        self.uiDefaultStyleAction.setObjectName("uiDefaultStyleAction")
        self.uiBrowseRoutersAction = QtWidgets.QAction(MainWindow)
        icon23 = QtGui.QIcon()
        icon23.addPixmap(QtGui.QPixmap(":/icons/router.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon23.addPixmap(QtGui.QPixmap(":/icons/router-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiBrowseRoutersAction.setIcon(icon23)
        self.uiBrowseRoutersAction.setObjectName("uiBrowseRoutersAction")
        self.uiBrowseSwitchesAction = QtWidgets.QAction(MainWindow)
        icon24 = QtGui.QIcon()
        icon24.addPixmap(QtGui.QPixmap(":/icons/switch.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon24.addPixmap(QtGui.QPixmap(":/icons/switch-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiBrowseSwitchesAction.setIcon(icon24)
        self.uiBrowseSwitchesAction.setObjectName("uiBrowseSwitchesAction")
        self.uiBrowseEndDevicesAction = QtWidgets.QAction(MainWindow)
        icon25 = QtGui.QIcon()
        icon25.addPixmap(QtGui.QPixmap(":/icons/PC.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon25.addPixmap(QtGui.QPixmap(":/icons/PC-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiBrowseEndDevicesAction.setIcon(icon25)
        self.uiBrowseEndDevicesAction.setObjectName("uiBrowseEndDevicesAction")
        self.uiBrowseSecurityDevicesAction = QtWidgets.QAction(MainWindow)
        icon26 = QtGui.QIcon()
        icon26.addPixmap(QtGui.QPixmap(":/icons/firewall.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon26.addPixmap(QtGui.QPixmap(":/icons/firewall-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiBrowseSecurityDevicesAction.setIcon(icon26)
        self.uiBrowseSecurityDevicesAction.setObjectName("uiBrowseSecurityDevicesAction")
        self.uiBrowseAllDevicesAction = QtWidgets.QAction(MainWindow)
        icon27 = QtGui.QIcon()
        icon27.addPixmap(QtGui.QPixmap(":/icons/browse-all-icons.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon27.addPixmap(QtGui.QPixmap(":/icons/browse-all-icons-hover.png"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        self.uiBrowseAllDevicesAction.setIcon(icon27)
        self.uiBrowseAllDevicesAction.setObjectName("uiBrowseAllDevicesAction")
        self.uiAddLinkAction = QtWidgets.QAction(MainWindow)
        self.uiAddLinkAction.setCheckable(True)
        icon28 = QtGui.QIcon()
        icon28.addPixmap(QtGui.QPixmap(":/icons/connection-new.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon28.addPixmap(QtGui.QPixmap(":/icons/connection-new-hover.svg"), QtGui.QIcon.Active, QtGui.QIcon.Off)
        icon28.addPixmap(QtGui.QPixmap(":/icons/cancel-connection.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        icon28.addPixmap(QtGui.QPixmap(":/icons/cancel-connection.svg"), QtGui.QIcon.Active, QtGui.QIcon.On)
        self.uiAddLinkAction.setIcon(icon28)
        self.uiAddLinkAction.setObjectName("uiAddLinkAction")
        self.uiGettingStartedAction = QtWidgets.QAction(MainWindow)
        self.uiGettingStartedAction.setObjectName("uiGettingStartedAction")
        self.uiLabInstructionsAction = QtWidgets.QAction(MainWindow)
        self.uiLabInstructionsAction.setObjectName("uiLabInstructionsAction")
        self.uiFitInViewAction = QtWidgets.QAction(MainWindow)
        self.uiFitInViewAction.setObjectName("uiFitInViewAction")
        self.uiExportProjectAction = QtWidgets.QAction(MainWindow)
        self.uiExportProjectAction.setVisible(False)
        self.uiExportProjectAction.setObjectName("uiExportProjectAction")
        self.uiImportProjectAction = QtWidgets.QAction(MainWindow)
        self.uiImportProjectAction.setVisible(False)
        self.uiImportProjectAction.setObjectName("uiImportProjectAction")
        self.uiMoveLocalProjectToCloudAction = QtWidgets.QAction(MainWindow)
        self.uiMoveLocalProjectToCloudAction.setVisible(False)
        self.uiMoveLocalProjectToCloudAction.setObjectName("uiMoveLocalProjectToCloudAction")
        self.uiMoveCloudProjectToLocalAction = QtWidgets.QAction(MainWindow)
        self.uiMoveCloudProjectToLocalAction.setVisible(False)
        self.uiMoveCloudProjectToLocalAction.setObjectName("uiMoveCloudProjectToLocalAction")
        self.uiDarkStyleAction = QtWidgets.QAction(MainWindow)
        self.uiDarkStyleAction.setObjectName("uiDarkStyleAction")
        self.uiActionFullscreen = QtWidgets.QAction(MainWindow)
        self.uiActionFullscreen.setObjectName("uiActionFullscreen")
        self.uiVPCSAction = QtWidgets.QAction(MainWindow)
        self.uiVPCSAction.setObjectName("uiVPCSAction")
        self.uiDownloadRemoteProject = QtWidgets.QAction(MainWindow)
        icon29 = QtGui.QIcon()
        icon29.addPixmap(QtGui.QPixmap(":/classic_icons/save-as-project.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiDownloadRemoteProject.setIcon(icon29)
        self.uiDownloadRemoteProject.setObjectName("uiDownloadRemoteProject")
        self.uiQemuImgWizardAction = QtWidgets.QAction(MainWindow)
        icon30 = QtGui.QIcon()
        icon30.addPixmap(QtGui.QPixmap(":/icons/qemu.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.uiQemuImgWizardAction.setIcon(icon30)
        self.uiQemuImgWizardAction.setObjectName("uiQemuImgWizardAction")
        self.uiSetupWizard = QtWidgets.QAction(MainWindow)
        self.uiSetupWizard.setMenuRole(QtWidgets.QAction.NoRole)
        self.uiSetupWizard.setObjectName("uiSetupWizard")
        self.uiIOUVMConverterAction = QtWidgets.QAction(MainWindow)
        self.uiIOUVMConverterAction.setObjectName("uiIOUVMConverterAction")
        self.uiOpenApplianceAction = QtWidgets.QAction(MainWindow)
        self.uiOpenApplianceAction.setIcon(icon1)
        self.uiOpenApplianceAction.setObjectName("uiOpenApplianceAction")
        self.uiExportDebugInformationAction = QtWidgets.QAction(MainWindow)
        self.uiExportDebugInformationAction.setObjectName("uiExportDebugInformationAction")
        self.uiDoctorAction = QtWidgets.QAction(MainWindow)
        self.uiDoctorAction.setObjectName("uiDoctorAction")
        self.uiAcademyAction = QtWidgets.QAction(MainWindow)
        self.uiAcademyAction.setObjectName("uiAcademyAction")
        self.uiEditMenu.addAction(self.uiSelectAllAction)
        self.uiEditMenu.addAction(self.uiSelectNoneAction)
        self.uiEditMenu.addSeparator()
        self.uiEditMenu.addAction(self.uiPreferencesAction)
        self.uiFileMenu.addAction(self.uiNewProjectAction)
        self.uiFileMenu.addAction(self.uiOpenProjectAction)
        self.uiFileMenu.addAction(self.uiSaveProjectAction)
        self.uiFileMenu.addAction(self.uiSaveProjectAsAction)
        self.uiFileMenu.addAction(self.uiExportProjectAction)
        self.uiFileMenu.addAction(self.uiImportProjectAction)
        self.uiFileMenu.addSeparator()
        self.uiFileMenu.addAction(self.uiMoveLocalProjectToCloudAction)
        self.uiFileMenu.addAction(self.uiMoveCloudProjectToLocalAction)
        self.uiFileMenu.addAction(self.uiDownloadRemoteProject)
        self.uiFileMenu.addSeparator()
        self.uiFileMenu.addAction(self.uiOpenApplianceAction)
        self.uiFileMenu.addAction(self.uiImportExportConfigsAction)
        self.uiFileMenu.addAction(self.uiScreenshotAction)
        self.uiFileMenu.addAction(self.uiSnapshotAction)
        self.uiFileMenu.addSeparator()
        self.uiFileMenu.addAction(self.uiQuitAction)
        self.uiHelpMenu.addAction(self.uiOnlineHelpAction)
        self.uiHelpMenu.addAction(self.uiCheckForUpdateAction)
        self.uiHelpMenu.addAction(self.uiSetupWizard)
        self.uiHelpMenu.addAction(self.uiAcademyAction)
        self.uiHelpMenu.addAction(self.uiDoctorAction)
        self.uiHelpMenu.addAction(self.uiExportDebugInformationAction)
        self.uiHelpMenu.addAction(self.uiAboutQtAction)
        self.uiHelpMenu.addAction(self.uiAboutAction)
        self.uiViewMenu.addAction(self.uiActionFullscreen)
        self.uiViewMenu.addSeparator()
        self.uiViewMenu.addAction(self.uiZoomInAction)
        self.uiViewMenu.addAction(self.uiZoomOutAction)
        self.uiViewMenu.addAction(self.uiZoomResetAction)
        self.uiViewMenu.addAction(self.uiFitInViewAction)
        self.uiViewMenu.addSeparator()
        self.uiViewMenu.addAction(self.uiShowLayersAction)
        self.uiViewMenu.addAction(self.uiResetPortLabelsAction)
        self.uiViewMenu.addAction(self.uiShowPortNamesAction)
        self.uiViewMenu.addSeparator()
        self.uiViewMenu.addAction(self.uiDocksMenu.menuAction())
        self.uiControlMenu.addAction(self.uiStartAllAction)
        self.uiControlMenu.addAction(self.uiSuspendAllAction)
        self.uiControlMenu.addAction(self.uiStopAllAction)
        self.uiControlMenu.addAction(self.uiReloadAllAction)
        self.uiControlMenu.addAction(self.uiAuxConsoleAllAction)
        self.uiControlMenu.addAction(self.uiConsoleAllAction)
        self.uiAnnotateMenu.addAction(self.uiAddNoteAction)
        self.uiAnnotateMenu.addAction(self.uiInsertImageAction)
        self.uiAnnotateMenu.addAction(self.uiDrawRectangleAction)
        self.uiAnnotateMenu.addAction(self.uiDrawEllipseAction)
        self.uiToolsMenu.addAction(self.uiVPCSAction)
        self.uiToolsMenu.addAction(self.uiIOUVMConverterAction)
        self.uiMenuBar.addAction(self.uiFileMenu.menuAction())
        self.uiMenuBar.addAction(self.uiEditMenu.menuAction())
        self.uiMenuBar.addAction(self.uiViewMenu.menuAction())
        self.uiMenuBar.addAction(self.uiControlMenu.menuAction())
        self.uiMenuBar.addAction(self.uiDeviceMenu.menuAction())
        self.uiMenuBar.addAction(self.uiAnnotateMenu.menuAction())
        self.uiMenuBar.addAction(self.uiToolsMenu.menuAction())
        self.uiMenuBar.addAction(self.uiHelpMenu.menuAction())
        self.uiGeneralToolBar.addAction(self.uiNewProjectAction)
        self.uiGeneralToolBar.addAction(self.uiOpenProjectAction)
        self.uiGeneralToolBar.addAction(self.uiSaveProjectAction)
        self.uiBrowsersToolBar.addAction(self.uiBrowseRoutersAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiBrowsersToolBar.addAction(self.uiBrowseSwitchesAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiBrowsersToolBar.addAction(self.uiBrowseEndDevicesAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiBrowsersToolBar.addAction(self.uiBrowseSecurityDevicesAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiBrowsersToolBar.addAction(self.uiBrowseAllDevicesAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiBrowsersToolBar.addAction(self.uiAddLinkAction)
        self.uiBrowsersToolBar.addSeparator()
        self.uiControlToolBar.addAction(self.uiSnapshotAction)
        self.uiControlToolBar.addAction(self.uiShowPortNamesAction)
        self.uiControlToolBar.addAction(self.uiConsoleAllAction)
        self.uiControlToolBar.addSeparator()
        self.uiControlToolBar.addAction(self.uiStartAllAction)
        self.uiControlToolBar.addAction(self.uiSuspendAllAction)
        self.uiControlToolBar.addAction(self.uiStopAllAction)
        self.uiControlToolBar.addAction(self.uiReloadAllAction)
        self.uiAnnotationToolBar.addAction(self.uiAddNoteAction)
        self.uiAnnotationToolBar.addAction(self.uiInsertImageAction)
        self.uiAnnotationToolBar.addAction(self.uiDrawRectangleAction)
        self.uiAnnotationToolBar.addAction(self.uiDrawEllipseAction)
        self.uiAnnotationToolBar.addAction(self.uiZoomInAction)
        self.uiAnnotationToolBar.addAction(self.uiZoomOutAction)
        self.uiAnnotationToolBar.addAction(self.uiScreenshotAction)

        self.retranslateUi(MainWindow)
        self.uiQuitAction.triggered.connect(MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.uiGraphicsView, self.uiNodesView)
        MainWindow.setTabOrder(self.uiNodesView, self.uiConsoleTextEdit)
        MainWindow.setTabOrder(self.uiConsoleTextEdit, self.uiTopologySummaryTreeWidget)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.uiEditMenu.setTitle(_translate("MainWindow", "&Edit"))
        self.uiFileMenu.setTitle(_translate("MainWindow", "&File"))
        self.uiHelpMenu.setTitle(_translate("MainWindow", "&Help"))
        self.uiViewMenu.setTitle(_translate("MainWindow", "&View"))
        self.uiDocksMenu.setTitle(_translate("MainWindow", "Docks"))
        self.uiControlMenu.setTitle(_translate("MainWindow", "Control"))
        self.uiAnnotateMenu.setTitle(_translate("MainWindow", "Annotate"))
        self.uiDeviceMenu.setTitle(_translate("MainWindow", "Device"))
        self.uiToolsMenu.setTitle(_translate("MainWindow", "&Tools"))
        self.uiGeneralToolBar.setWindowTitle(_translate("MainWindow", "General"))
        self.uiNodesDockWidget.setWindowTitle(_translate("MainWindow", "Node Types"))
        self.uiNodesView.setToolTip(_translate("MainWindow", "Drag a node to the workspace (Press SHIFT while dragging to add multiple identical nodes)."))
        self.uiNodesView.headerItem().setText(0, _translate("MainWindow", "1"))
        self.uiBrowsersToolBar.setWindowTitle(_translate("MainWindow", "Devices"))
        self.uiControlToolBar.setWindowTitle(_translate("MainWindow", "Emulation"))
        self.uiConsoleDockWidget.setWindowTitle(_translate("MainWindow", "Console"))
        self.uiAnnotationToolBar.setWindowTitle(_translate("MainWindow", "Drawing"))
        self.uiTopologySummaryDockWidget.setWindowTitle(_translate("MainWindow", "Topology Summary"))
        self.uiTopologySummaryTreeWidget.headerItem().setText(0, _translate("MainWindow", "1"))
        self.uiServerSummaryDockWidget.setWindowTitle(_translate("MainWindow", "Servers Summary"))
        self.uiAboutAction.setText(_translate("MainWindow", "&About"))
        self.uiAboutAction.setStatusTip(_translate("MainWindow", "About"))
        self.uiQuitAction.setText(_translate("MainWindow", "&Quit"))
        self.uiQuitAction.setStatusTip(_translate("MainWindow", "Quit"))
        self.uiQuitAction.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.uiOpenProjectAction.setText(_translate("MainWindow", "&Open Project"))
        self.uiOpenProjectAction.setToolTip(_translate("MainWindow", "Open project"))
        self.uiOpenProjectAction.setStatusTip(_translate("MainWindow", "Open project"))
        self.uiOpenProjectAction.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.uiSaveProjectAction.setText(_translate("MainWindow", "&Save project"))
        self.uiSaveProjectAction.setToolTip(_translate("MainWindow", "Save project"))
        self.uiSaveProjectAction.setStatusTip(_translate("MainWindow", "Save project"))
        self.uiSaveProjectAction.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.uiOnlineHelpAction.setText(_translate("MainWindow", "&Online help"))
        self.uiOnlineHelpAction.setToolTip(_translate("MainWindow", "Online help"))
        self.uiOnlineHelpAction.setStatusTip(_translate("MainWindow", "Online Help"))
        self.uiScreenshotAction.setText(_translate("MainWindow", "Take a screenshot"))
        self.uiScreenshotAction.setToolTip(_translate("MainWindow", "Take a screenshot"))
        self.uiScreenshotAction.setStatusTip(_translate("MainWindow", "Take a screenshot"))
        self.uiStartAllAction.setText(_translate("MainWindow", "Start/Resume all devices"))
        self.uiStartAllAction.setToolTip(_translate("MainWindow", "Start/Resume all devices"))
        self.uiStartAllAction.setStatusTip(_translate("MainWindow", "Start/Resume all devices"))
        self.uiStopAllAction.setText(_translate("MainWindow", "Stop all devices"))
        self.uiStopAllAction.setToolTip(_translate("MainWindow", "Stop all devices"))
        self.uiStopAllAction.setStatusTip(_translate("MainWindow", "Stop all devices"))
        self.uiConsoleAllAction.setText(_translate("MainWindow", "Console connect to all devices"))
        self.uiConsoleAllAction.setToolTip(_translate("MainWindow", "Console connect to all devices"))
        self.uiConsoleAllAction.setStatusTip(_translate("MainWindow", "Console to all devices"))
        self.uiAboutQtAction.setText(_translate("MainWindow", "About &Qt"))
        self.uiAboutQtAction.setStatusTip(_translate("MainWindow", "About Qt"))
        self.uiZoomInAction.setText(_translate("MainWindow", "Zoom in"))
        self.uiZoomInAction.setToolTip(_translate("MainWindow", "Zoom in"))
        self.uiZoomInAction.setStatusTip(_translate("MainWindow", "Zoom in"))
        self.uiZoomInAction.setShortcut(_translate("MainWindow", "Ctrl++"))
        self.uiZoomOutAction.setText(_translate("MainWindow", "Zoom out"))
        self.uiZoomOutAction.setToolTip(_translate("MainWindow", "Zoom out"))
        self.uiZoomOutAction.setStatusTip(_translate("MainWindow", "Zoom out"))
        self.uiZoomOutAction.setShortcut(_translate("MainWindow", "Ctrl+-"))
        self.uiZoomResetAction.setText(_translate("MainWindow", "Zoom reset"))
        self.uiZoomResetAction.setStatusTip(_translate("MainWindow", "Zoom reset"))
        self.uiZoomResetAction.setShortcut(_translate("MainWindow", "Ctrl+0"))
        self.uiSelectAllAction.setText(_translate("MainWindow", "Select &all"))
        self.uiSelectAllAction.setStatusTip(_translate("MainWindow", "Select All"))
        self.uiSelectAllAction.setShortcut(_translate("MainWindow", "Ctrl+A"))
        self.uiSelectNoneAction.setText(_translate("MainWindow", "Select &none"))
        self.uiSelectNoneAction.setStatusTip(_translate("MainWindow", "Select None"))
        self.uiSelectNoneAction.setShortcut(_translate("MainWindow", "Ctrl+Shift+A"))
        self.uiPreferencesAction.setText(_translate("MainWindow", "&Preferences..."))
        self.uiPreferencesAction.setStatusTip(_translate("MainWindow", "Preferences"))
        self.uiPreferencesAction.setShortcut(_translate("MainWindow", "Ctrl+Shift+P"))
        self.uiSuspendAllAction.setText(_translate("MainWindow", "Suspend all devices"))
        self.uiSuspendAllAction.setToolTip(_translate("MainWindow", "Suspend all devices"))
        self.uiSuspendAllAction.setStatusTip(_translate("MainWindow", "Suspend all devices"))
        self.uiAddNoteAction.setText(_translate("MainWindow", "Add note"))
        self.uiAddNoteAction.setToolTip(_translate("MainWindow", "Add a note"))
        self.uiAddNoteAction.setStatusTip(_translate("MainWindow", "Add a note"))
        self.uiNewProjectAction.setText(_translate("MainWindow", "&New blank project"))
        self.uiNewProjectAction.setIconText(_translate("MainWindow", "New Project"))
        self.uiNewProjectAction.setToolTip(_translate("MainWindow", "New blank project"))
        self.uiNewProjectAction.setStatusTip(_translate("MainWindow", "New blank project"))
        self.uiNewProjectAction.setShortcut(_translate("MainWindow", "Ctrl+N"))
        self.uiImportExportConfigsAction.setText(_translate("MainWindow", "&Import/Export device configs"))
        self.uiImportExportConfigsAction.setToolTip(_translate("MainWindow", "Import/Export device configs"))
        self.uiImportExportConfigsAction.setStatusTip(_translate("MainWindow", "Import/Export device configs"))
        self.uiInsertImageAction.setText(_translate("MainWindow", "Insert picture"))
        self.uiInsertImageAction.setToolTip(_translate("MainWindow", "Insert a picture"))
        self.uiInsertImageAction.setStatusTip(_translate("MainWindow", "Insert a picture"))
        self.uiDrawRectangleAction.setText(_translate("MainWindow", "Draw rectangle"))
        self.uiDrawRectangleAction.setToolTip(_translate("MainWindow", "Draw a rectangle"))
        self.uiDrawRectangleAction.setStatusTip(_translate("MainWindow", "Draw a rectangle"))
        self.uiDrawEllipseAction.setText(_translate("MainWindow", "Draw ellipse"))
        self.uiDrawEllipseAction.setToolTip(_translate("MainWindow", "Draw an ellipse"))
        self.uiDrawEllipseAction.setStatusTip(_translate("MainWindow", "Draw an ellipse"))
        self.uiShowPortNamesAction.setText(_translate("MainWindow", "Show/Hide interface labels"))
        self.uiShowPortNamesAction.setToolTip(_translate("MainWindow", "Show/Hide interface labels"))
        self.uiShowPortNamesAction.setStatusTip(_translate("MainWindow", "Show/Hide interface labels"))
        self.uiSnapshotAction.setText(_translate("MainWindow", "Manage snapshots"))
        self.uiSnapshotAction.setToolTip(_translate("MainWindow", "Manage snapshots"))
        self.uiSnapshotAction.setStatusTip(_translate("MainWindow", "Manage snapshots"))
        self.uiShowLayersAction.setText(_translate("MainWindow", "Show layers"))
        self.uiShowLayersAction.setStatusTip(_translate("MainWindow", "Show layers"))
        self.uiSaveProjectAsAction.setText(_translate("MainWindow", "&Save project as..."))
        self.uiSaveProjectAsAction.setToolTip(_translate("MainWindow", "Save project as..."))
        self.uiSaveProjectAsAction.setStatusTip(_translate("MainWindow", "Save project as..."))
        self.uiReloadAllAction.setText(_translate("MainWindow", "Reload all devices"))
        self.uiReloadAllAction.setToolTip(_translate("MainWindow", "Reload all devices"))
        self.uiReloadAllAction.setStatusTip(_translate("MainWindow", "Reload all devices"))
        self.uiAuxConsoleAllAction.setText(_translate("MainWindow", "Console connect via AUX to all devices"))
        self.uiAuxConsoleAllAction.setToolTip(_translate("MainWindow", "Console connect via AUX to all devices"))
        self.uiAuxConsoleAllAction.setStatusTip(_translate("MainWindow", "Console AUX to all devices"))
        self.uiResetPortLabelsAction.setText(_translate("MainWindow", "Reset interface labels"))
        self.uiResetPortLabelsAction.setToolTip(_translate("MainWindow", "Reset interface labels"))
        self.uiResetPortLabelsAction.setStatusTip(_translate("MainWindow", "Reset Interface Labels"))
        self.uiCheckForUpdateAction.setText(_translate("MainWindow", "Check for &Update"))
        self.uiCheckForUpdateAction.setStatusTip(_translate("MainWindow", "Check for Update"))
        self.uiEnergySavingStyleAction.setText(_translate("MainWindow", "Energy Saving"))
        self.uiEnergySavingStyleAction.setStatusTip(_translate("MainWindow", "Energy Saving Mode"))
        self.uiDefaultStyleAction.setText(_translate("MainWindow", "Default"))
        self.uiBrowseRoutersAction.setText(_translate("MainWindow", "Browse Routers"))
        self.uiBrowseRoutersAction.setIconText(_translate("MainWindow", "Browse Routers"))
        self.uiBrowseRoutersAction.setToolTip(_translate("MainWindow", "Browse Routers"))
        self.uiBrowseRoutersAction.setStatusTip(_translate("MainWindow", "Browse Routers"))
        self.uiBrowseSwitchesAction.setText(_translate("MainWindow", "Browse Switches"))
        self.uiBrowseSwitchesAction.setIconText(_translate("MainWindow", "Browse Switches"))
        self.uiBrowseSwitchesAction.setToolTip(_translate("MainWindow", "Browse Switches"))
        self.uiBrowseSwitchesAction.setStatusTip(_translate("MainWindow", "Browse Switches"))
        self.uiBrowseEndDevicesAction.setText(_translate("MainWindow", "Browse End Devices"))
        self.uiBrowseEndDevicesAction.setIconText(_translate("MainWindow", "Browse End Devices"))
        self.uiBrowseEndDevicesAction.setToolTip(_translate("MainWindow", "Browse End Devices"))
        self.uiBrowseEndDevicesAction.setStatusTip(_translate("MainWindow", "Browse End Devices"))
        self.uiBrowseSecurityDevicesAction.setText(_translate("MainWindow", "Browse Security Devices"))
        self.uiBrowseSecurityDevicesAction.setIconText(_translate("MainWindow", "Browse Security Devices"))
        self.uiBrowseSecurityDevicesAction.setToolTip(_translate("MainWindow", "Browse Security Devices"))
        self.uiBrowseSecurityDevicesAction.setStatusTip(_translate("MainWindow", "Browse Security Devices"))
        self.uiBrowseAllDevicesAction.setText(_translate("MainWindow", "Browse all devices"))
        self.uiBrowseAllDevicesAction.setToolTip(_translate("MainWindow", "Browse all devices"))
        self.uiBrowseAllDevicesAction.setStatusTip(_translate("MainWindow", "Browse all devices"))
        self.uiAddLinkAction.setText(_translate("MainWindow", "Add a link"))
        self.uiAddLinkAction.setToolTip(_translate("MainWindow", "Add a link"))
        self.uiAddLinkAction.setStatusTip(_translate("MainWindow", "Add a link"))
        self.uiGettingStartedAction.setText(_translate("MainWindow", "Getting started"))
        self.uiGettingStartedAction.setToolTip(_translate("MainWindow", "Show GNS3 news"))
        self.uiLabInstructionsAction.setText(_translate("MainWindow", "&Lab instructions"))
        self.uiFitInViewAction.setText(_translate("MainWindow", "Fit in view"))
        self.uiExportProjectAction.setText(_translate("MainWindow", "Backup project to cloud"))
        self.uiImportProjectAction.setText(_translate("MainWindow", "Restore backup from cloud"))
        self.uiMoveLocalProjectToCloudAction.setText(_translate("MainWindow", "Move local project to cloud"))
        self.uiMoveCloudProjectToLocalAction.setText(_translate("MainWindow", "Move cloud project to local"))
        self.uiDarkStyleAction.setText(_translate("MainWindow", "Dark Style"))
        self.uiActionFullscreen.setText(_translate("MainWindow", "Fullscreen"))
        self.uiActionFullscreen.setShortcut(_translate("MainWindow", "Ctrl+F"))
        self.uiVPCSAction.setText(_translate("MainWindow", "&VPCS multi-host"))
        self.uiDownloadRemoteProject.setText(_translate("MainWindow", "Download remote project"))
        self.uiQemuImgWizardAction.setText(_translate("MainWindow", "Qemu image wizard"))
        self.uiSetupWizard.setText(_translate("MainWindow", "&Setup Wizard"))
        self.uiIOUVMConverterAction.setText(_translate("MainWindow", "IOU VM Converter"))
        self.uiOpenApplianceAction.setText(_translate("MainWindow", "Import appliance"))
        self.uiExportDebugInformationAction.setText(_translate("MainWindow", "Export debug information"))
        self.uiExportDebugInformationAction.setToolTip(_translate("MainWindow", "&Export debug information"))
        self.uiDoctorAction.setText(_translate("MainWindow", "GNS3 &Doctor"))
        self.uiAcademyAction.setText(_translate("MainWindow", "GNS3 &Academy"))

from ..topology_summary_view import TopologySummaryView
from ..graphics_view import GraphicsView
from ..server_summary_view import ServerSummaryView
from ..nodes_view import NodesView
from ..console_view import ConsoleView
from . import resources_rc
